package wxfunc

import (
	"crypto/sha1"
	"encoding/json"
	"fmt"
	"errors"
	"github.com/xlstudio/wxbizdatacrypt"
	"net/http"
)

/**
 * 常量
 * @param appid string 小程序的appid
 */
const appId = "wxe0d79550e7f33316"
const secret = "2bb0e3104b315def0db87c131bbc351f"

//加密数据解密算法

/**
	 * 检验数据的真实性，并且获取解密后的明文.
	 * @param encryptedData string 加密的用户数据
	 * @param iv string 与用户数据一同返回的初始向量
	 * @param sessionKey string 与用户数据一同返回的初始向量
     *
	 * @return map type.
*/
func DecryptData(encryptedData string, iv string, sessionKey string) (interface{}, error) {

	pc := wxbizdatacrypt.WxBizDataCrypt{AppId: appId, SessionKey: sessionKey}
	result, err := pc.Decrypt(encryptedData, iv, true) //第三个参数解释： 需要返回 JSON 数据类型时 使用 true, 需要返回 map 数据类型时 使用 false
	if err != nil {
		return nil, err
	}
	return result, nil
}


type WXLoginResp struct {
	OpenId string           `json:"openid"`
	SessionKey string       `json:"session_key"`
	UnionId string          `json:"unionid"`
	ErrCode int             `json:"errcode"`
	ErrMsg string           `json:"errmsg"`
}

// 这个函数以 code 作为输入, 返回调用微信接口得到的对象指针和异常情况
func WXLogin(code string) (*WXLoginResp, error) {
	url := "https://api.weixin.qq.com/sns/jscode2session?appid=%s&secret=%s&js_code=%s&grant_type=authorization_code"

	// 合成url, 这里的appId和secret是在微信公众平台上获取的
	url = fmt.Sprintf(url, appId, secret, code)

	// 创建http get请求
	resp,err := http.Get(url)
	if err != nil {
		return nil, err
	}
	defer resp.Body.Close()

	// 解析http请求中body 数据到我们定义的结构体中
	wxResp := WXLoginResp{}
	decoder := json.NewDecoder(resp.Body)
	if err := decoder.Decode(&wxResp); err != nil {
		return nil, err
	}

	// 判断微信接口返回的是否是一个异常情况
	if wxResp.ErrCode != 0 {
		return nil, errors.New(fmt.Sprintf("ErrCode:%s  ErrMsg:%s", wxResp.ErrCode, wxResp.ErrMsg))
	}

	return &wxResp, nil
}


// 校验微信返回的用户数据
func ValidateUserInfo(rawData, sessionKey, signature string) bool {
	signature2 := GetSha1(rawData + sessionKey)
	return signature == signature2
}
// SHA-1 加密
func GetSha1(str string) string {
	data := []byte(str)
	has := sha1.Sum(data)
	res := fmt.Sprintf("%x", has) //将[]byte转成16进制
	return res
}