package controllers

import (
	"SSW_WebPlatform/utils"
	"SSW_WebPlatform/web/middleware/glog"
	"SSW_WebPlatform/web/middleware/jwts"
	"SSW_WebPlatform/web/models"
	"SSW_WebPlatform/web/supports"
	"github.com/kataras/iris/v12"
	"strings"
	"time"
)

func GetCSurveysheetList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/CSurveysheet_controller.go"
	logs.Level = "info"
	logs.Function = "GetCSurveysheetList"
	logs.Message = "调查问卷查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.CSurveysheet
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	data.Descr = ctx.URLParam("descr")
	data.Status, _ = ctx.URLParamInt("statusOption")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetCSurveysheet(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.CSurveysheet)
	me.Cid = user.Pid
	me.Cnr, _ = ctx.URLParamInt("cnr")
	me.Surveysheetid = ctx.URLParam("surveysheetid")
	level, _ := ctx.URLParamInt("level")
	result, err := me.SelectOne(level)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func GetCSurveysheetByUser(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.CSurveysheet)
	me.Cid, _ = ctx.URLParamInt("cid")
	me.Cnr, _ = ctx.URLParamInt("cid")
	me.Lastmodifyby = ctx.URLParam("userName")
	me.Surveysheetid = ctx.URLParam("surveysheetid")
	level, _ := ctx.URLParamInt("level")
	result, err := me.SelectOneInfo(level)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	//根据用户查询答案
	userResult := new(models.CSurveysheetResult)
	userResult.Cid, _ = ctx.URLParamInt("cid")
	userResult.Lastmodifyby = ctx.URLParam("userName")
	userResult.Surveysheetid = ctx.URLParam("surveysheetid")
	data, err := userResult.SurveysheetSearchByUser()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	cache := make(map[string][]models.CSurveysheetResultlst)
	for _, v := range data {
		key := strings.TrimSpace(v.Subjectid)
		value := v
		cache[key] = append(cache[key], value)
	}
	for kk, vv := range result.Valst {
		key := strings.TrimSpace(vv.Subjectid)
		val, ok := cache[key]
		if ok {
			if len(val) > 0 {
				var SelectedOptions []string
				var Subjectid string
				var Optioninput string
				for _, v := range val {
					Subjectid = v.Subjectid
					Optioninput = v.Optioninput
					SelectedOptions = append(SelectedOptions, v.SelectedOptions)
				}
				Options := strings.Join(SelectedOptions, ",")
				result.Valst[kk].QuestionNaireItem.Subjectid = Subjectid
				result.Valst[kk].QuestionNaireItem.Optioninput = Optioninput
				result.Valst[kk].QuestionNaireItem.Selected_options = Options
			}

		}
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertCSurveysheet(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Subject_controller.go"
	logs.Level = "info"
	logs.Function = "InsertCSurveysheet"
	logs.Message = "添加调查问卷"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.CSurveysheet)
	data.Lastmodifyby = user.Userid
	data.Createtime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodifytime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		logs := new(models.LeitServerLog)
		logs.File = "/controllers/Subject_controller.go"
		logs.Level = "debug"
		logs.Function = "InsertSubject"
		logs.Message = error.Error(err)
		logs.Operator = user.Userid
		logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
		logs.InsertRecord()
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Cid = user.Pid
	err := data.Add()
	if err != nil {
		glog.InfoExt("问题调研", "add err is :", err)
		supports.Error(ctx, iris.StatusBadRequest, "添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateCSurveysheet(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Subject_controller.go"
	logs.Level = "info"
	logs.Function = "UpdateSubject"
	logs.Message = "更新调查问卷"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.CSurveysheet)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastmodifyby = user.Userid
	err := data.Update()
	if err != nil {
		glog.InfoExt("问题调研", "update err is :", err)
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteCSurveysheet(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/Subject_controller.go"
	logs.Level = "info"
	logs.Function = "DeleteSubject"
	logs.Message = "删除调查问卷"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	me := new(models.CSurveysheet)
	me.Cid = user.Pid
	me.Cnr, _ = ctx.URLParamInt("cnr")
	me.Surveysheetid = ctx.URLParam("surveysheetid")
	err := me.Del()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}

func SelectArrCSurveysheet(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.CSurveysheet)
	//me.Cid = user.Pid
	result, err := me.SelectArr()
	if err != nil {

		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func StatisticData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}

	me := new(models.CSurveysheetResult)
	selectType := ctx.URLParam("selectType")
	if user.Role == "group" || user.Role == "admin" {
		cid, _ := ctx.URLParamInt("cid")
		if cid < 0 {
			me.Cid = user.Pid
		} else {
			me.Cid, _ = ctx.URLParamInt("cid")
		}

	} else {
		me.Cid = user.Pid
	}
	year := ctx.URLParam("year")
	if utils.ValueIsEmpty(year) {
		supports.Error(ctx, iris.StatusBadRequest, "参数不能空！", nil)
		return
	}
	unit := ctx.URLParam("unit")
	result, err := me.Statistic(selectType, year, unit)
	if err != nil {

		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//集团用户查看对比
func AllStatisticData(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.CSurveysheetResult)
	selectType := ctx.URLParam("selectType")
	year := ctx.URLParam("year")
	if utils.ValueIsEmpty(year) {
		supports.Error(ctx, iris.StatusBadRequest, "参数不能空！", nil)
		return
	}
	unit := ctx.URLParam("unit")
	cidStr := ctx.URLParam("cids")
	cids := strings.Split(cidStr, ",")
	result, err := me.AllStatistic(selectType, cids, year, unit)
	if err != nil {

		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}
