package controllers

import (
	"SSW_WebPlatform/utils"
	"SSW_WebPlatform/web/middleware/jwts"
	"SSW_WebPlatform/web/models"
	"SSW_WebPlatform/web/supports"
	"github.com/kataras/iris/v12"
	"time"
)

func GetSubjectCategoryList(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/SubjectCategory_controller.go"
	logs.Level = "info"
	logs.Function = "GetSubjectCategoryList"
	logs.Message = "问题分类查看"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	var data models.SubjectCategory
	var err error
	var pageSize = 10
	var pageIndex = 1

	if size := ctx.URLParam("pageSize"); size != "" {
		pageSize = utils.ValueToInt(size, 0)
	}

	if index := ctx.URLParam("pageIndex"); index != "" {
		pageIndex = utils.ValueToInt(index, 0)
	}
	data.Cid = user.Pid
	data.SubjectCategoryid = ctx.URLParam("subject_categoryid")
	data.SubjectCategoryid = ctx.URLParam("subject_categoryid")
	result, count, err := data.GetPage(pageSize, pageIndex)
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	res := make(map[string]interface{})
	res["data"] = result
	res["count"] = count
	res["pageIndex"] = pageIndex
	res["pageSize"] = pageSize
	supports.Ok(ctx, supports.OptionSuccess, res)
}

func GetSubjectCategory(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	me := new(models.SubjectCategory)
	me.Cid = user.Pid
	me.SubjectCategoryid = ctx.URLParam("subject_categoryid")
	result, err := me.SelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	result.Clipped()
	supports.Ok(ctx, supports.OptionSuccess, result)
}

func InsertSubjectCategory(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/SubjectCategory_controller.go"
	logs.Level = "info"
	logs.Function = "InsertSubjectCategory"
	logs.Message = "添加生产线"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.SubjectCategory)
	data.Lastmodifyby = user.Userid
	data.Createtime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodifytime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	if err := ctx.ReadJSON(data); err != nil {
		logs := new(models.LeitServerLog)
		logs.File = "/controllers/SubjectCategory_controller.go"
		logs.Level = "debug"
		logs.Function = "InsertSubjectCategory"
		logs.Message = error.Error(err)
		logs.Operator = user.Userid
		logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
		logs.InsertRecord()
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Cid = user.Pid
	err := data.Add()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, data)
}

func UpdateSubjectCategory(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/SubjectCategory_controller.go"
	logs.Level = "info"
	logs.Function = "UpdateSubjectCategory"
	logs.Message = "更新问题分类"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	data := new(models.SubjectCategory)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "抱歉未找到相关信息", nil)
		return
	}
	data.Lastmodifyby = user.Userid
	res := data.Update()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "更新失败", nil)
		return
	}

	supports.Ok(ctx, "修改成功", "")
}

func DeleteSubjectCategory(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	logs := new(models.LeitServerLog)
	logs.File = "/controllers/SubjectCategory_controller.go"
	logs.Level = "info"
	logs.Function = "DeleteSubjectCategory"
	logs.Message = "删除问题分类"
	logs.Operator = user.Userid
	logs.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	logs.InsertRecord()
	me := new(models.SubjectCategory)
	me.Cid = user.Pid
	me.SubjectCategoryid = ctx.URLParam("subject_categoryid")
	res := me.Del()
	if !res {
		supports.Error(ctx, iris.StatusBadRequest, "删除失败", nil)
		return
	}
	supports.Ok(ctx, "删除成功", "")
}
func AllSubjectCategory(ctx iris.Context) {
	user, ok := jwts.ParseToken(ctx)
	utils.TrimStruct(user, *user)
	if !ok {
		supports.Error(ctx, iris.StatusBadRequest, supports.ParseParamsFailur, nil)
		return
	}
	//日志
	log := new(models.LeitServerLog)
	log.File = "/controllers/WfApprovalmodel_controller.go"
	log.Level = "info"
	log.Function = "SubjectCategory"
	log.Message = "查询问题类型"
	log.Operator = user.Userid
	log.TimeStamp = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	log.InsertRecord()
	me := new(models.SubjectCategory)
	me.Cid = user.Pid
	data, err := me.SelectArr()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "失败", nil)
		return
	}
	supports.Ok(ctx, "成功", data)
}
