package wxSmall

import (
	"SSW_WebPlatform/utils"
	"SSW_WebPlatform/web/middleware/glog"
	"SSW_WebPlatform/web/models"
	"SSW_WebPlatform/web/models/wxSmall"
	"SSW_WebPlatform/web/supports"
	"github.com/kataras/iris/v12"
	"strings"
	"time"
)

//查询楼栋信息
func GetBuildings(ctx iris.Context) {
	me := new(models.Buildingtab)
	me.Cid, _ = ctx.URLParamInt("cid")
	if utils.ValueIsEmpty(me.Cid) {
		supports.Error(ctx, iris.StatusBadRequest, "项目编号不能为空！", nil)
		return
	}
	result, err := me.SelectAll()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//查询调查问卷
func GetWxCSurveysheet(ctx iris.Context) {
	me := new(models.CSurveysheet)
	me.Cid, _ = ctx.URLParamInt("cid")
	me.Cnr, _ = ctx.URLParamInt("cid")
	me.Surveysheetid = ctx.URLParam("surveysheetid")
	result, err := me.WxSelectOne()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	//查询缓存
	wx := new(wxSmall.WxUserCache)
	wx.Cid, _ = ctx.URLParamInt("cid")
	wx.Uid, _ = ctx.URLParamInt("uid")
	wx.Surveysheetid = ctx.URLParam("surveysheetid")
	data, err := wx.SelectCacheInfo()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, err.Error(), nil)
		return
	}
	cache := make(map[string]wxSmall.WxCacheSurveysheetResultlst)
	for _, v := range data {
		key := strings.TrimSpace(v.Subjectid)
		value := v
		cache[key] = value
	}
	for kk, vv := range result.Valst {
		key := strings.TrimSpace(vv.Subjectid)
		val, ok := cache[key]
		if ok {
			result.Valst[kk].QuestionNaireItem.Subjectid = val.Subjectid
			result.Valst[kk].QuestionNaireItem.Optioninput = val.Optioninput
			result.Valst[kk].QuestionNaireItem.Selected_options = val.SelectedOptions
		}
	}
	supports.Ok(ctx, supports.OptionSuccess, result)
}

//提交试卷
func InsertWXCSurveysheetResult(ctx iris.Context) {
	data := new(models.CSurveysheetResult)
	if err := ctx.ReadJSON(data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}
	data.Lastmodifyby = "wx"
	data.Createtime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	data.Lastmodifytime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	err := data.Add()
	if err != nil {
		glog.InfoExt("问题调研", "add err is :", err)
		supports.Error(ctx, iris.StatusBadRequest, "添加失败！", nil)
		return
	}
	//删除缓存
	cache := new(wxSmall.WxUserCache)
	cache.Uid = data.Uid
	cache.Surveysheetid = data.Surveysheetid
	cache.Cid = data.Cid
	cache.DelCache()
	supports.Ok(ctx, supports.OptionSuccess, "")
}

//缓存试卷
func AddWxCacheSurveysheetResult(ctx iris.Context) {
	var data models.CSurveysheetResult
	if err := ctx.ReadJSON(&data); err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "json解析错误", nil)
		return
	}

	//第一步:查询缓存用户关联表，有就更新，没有就插入
	cache := new(wxSmall.WxUserCache)
	cache.Uid = data.Uid
	cache.Surveysheetid = data.Surveysheetid
	cache.Cid = data.Cid
	cache.Status = 0
	cache.Createtime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	cache.Lastmodifytime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")

	_, err := cache.LoadInfo()
	if err != nil {
		supports.Error(ctx, iris.StatusBadRequest, "查询缓存失败", nil)
		return
	}
	//第二步：题目信息，插入缓存表里，有就更新，没有就插入
	wxsurvey := new(wxSmall.WxCacheSurveysheetResult)
	wxsurvey.Lastmodifyby = "wx"
	wxsurvey.Createtime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	wxsurvey.Lastmodifytime = utils.TimeFormat(time.Now(), "yyyyMMddHHmmss")
	wxsurvey.Surveynr = data.Surveynr
	wxsurvey.Uid = data.Uid
	err = wxsurvey.Add(data)
	if err != nil {
		glog.InfoExt("问题调研", "add err is :", err)
		supports.Error(ctx, iris.StatusBadRequest, "添加失败！", nil)
		return
	}
	supports.Ok(ctx, supports.OptionSuccess, "")
}
