package models

import (
	"SSW_WebPlatform/db"
	"SSW_WebPlatform/utils"
	"SSW_WebPlatform/web/middleware/glog"
	"errors"
	"fmt"
	"sort"
	"strings"
	"time"
	"xorm.io/core"
)

type CSurveysheetResult struct {
	Cid                 int    `json:"cid" xorm:"not null pk comment('公司ID') INT(11)"`
	Cnr                 int    `json:"cnr" xorm:"not null pk comment('小区编号') INT(11)"`
	Surveysheetid       string `json:"surveysheetid" xorm:"not null pk VARCHAR(40)"`
	Surveynr            int    `json:"surveynr" xorm:"not null pk comment('调研问卷流水号') INT(11)"`
	Surveyperson        string `json:"surveyperson" xorm:"default 'NULL' comment('指定调研人') VARCHAR(40)"`
	Surveysamplecount   int    `json:"surveysamplecount" xorm:"not null comment('调研取样数') INT(11)"`
	Surveyfinishedcount int    `json:"surveyfinishedcount" xorm:"not null comment('调研问卷实际已完成数') INT(11)"`
	Buildingid          string `json:"buildingid" xorm:"VARCHAR(100)"`
	Unit                string `json:"unit" xorm:"VARCHAR(40)"`
	Room                string `json:"room" xorm:"VARCHAR(40)"`
	Planstartdate       string `json:"planstartdate" xorm:"default 'NULL' VARCHAR(8)"`
	Planendate          string `json:"planendate" xorm:"default 'NULL' VARCHAR(8)"`
	Actstartdate        string `json:"actstartdate" xorm:"default 'NULL' VARCHAR(8)"`
	Actenddate          string `json:"actenddate" xorm:"default 'NULL' VARCHAR(8)"`
	Status              int    `json:"status" xorm:"not null comment('状态') INT(11)"`
	Remark              string `json:"remark" xorm:"default 'NULL' comment('备注') VARCHAR(255)"`
	// 被访问者性别 male 和female
	Sex string `json:"sex" xorm:"VARCHAR(10)"`
	// 被访问者类型 业主1 租户2
	PersonType int `json:"persontype" xorm:"'persontype' INT(1)"`
	// 被访问者年龄 1-22岁以下 2-22到59岁 3-60岁以上
	Age int `json:"age" xorm:"INT(1)"`
	// 被访问者居住时间 1-1年以内 2-1到3年 2-3到5年 3-5年以上
	LiveTime int `json:"livetime" xorm:"'livetime' INT(1)"`
	// 被访问者姓名
	PersonName string `json:"personname" xorm:"'personname' VARCHAR(40)"`
	//// 调查日期
	//SurveyDate string `json:"surveydate" xorm:"'surveydate' VARCHAR(20)"`
	//// 调查时间 小时
	//SurveyHour int `json:"surveyhour" xorm:"'surveyhour' INT(2)"`

	Signature      string                  `json:"signature" xorm:"default 'NULL' comment('电子签名') VARCHAR(255)"`
	Createtime     string                  `json:"createtime" xorm:"default 'NULL' comment('创建时间') VARCHAR(40)"`
	Lastmodifytime string                  `json:"lastmodifytime" xorm:"default 'NULL' comment('最近更新时间') VARCHAR(40)"`
	Lastmodifyby   string                  `json:"lastmodifyby" xorm:"default 'NULL' comment('修改人员') VARCHAR(40)"`
	Uid            int                     `json:"uid" xorm:"-"`
	Valst          []CSurveysheetResultlst `json:"valst" xorm:"-"`
}

func (t *CSurveysheetResult) TableName() string {
	return "c_surveysheet_result"
}

// 清除string字段的右侧空格
func (t *CSurveysheetResult) Clipped() {
	utils.TrimStruct(t, *t)
}

//添加
//增
func (t *CSurveysheetResult) Add() error {
	engine := db.MasterEngine()
	//判断是否提交
	oneSurvey := new(CSurveysheetResult)
	ok, err := engine.Table("c_surveysheet_result").Where("cid = ? and surveysheetid = ? and lastmodifyby = ?", t.Cid, t.Surveysheetid, t.Lastmodifyby).Get(oneSurvey)
	if err != nil {
		return err
	}
	if ok {
		return errors.New("不能重复提交！")
	}
	session := engine.NewSession()
	defer session.Close()
	// add Begin() before any action
	err = session.Begin()
	if err != nil {
		return err
	}
	snr := new(Snrtab)
	snr.Cid = t.Cid
	id, err := snr.GetNextSnr("Surveynr")
	if err != nil {
		session.Rollback()
		return err
	}
	//查询信息
	subject := new(CSurveysheet)
	subject.Cid = t.Cid
	subject.Surveysheetid = t.Surveysheetid
	subject.Cnr = t.Cnr
	_, err = engine.Table("c_surveysheet").ID(core.PK{subject.Cid, subject.Cnr, subject.Surveysheetid}).Get(subject)
	if err != nil {
		session.Rollback()
		return err
	}
	//更新调研个数
	CSurveysheet := new(CSurveysheet)
	CSurveysheet.Cid = t.Cid
	CSurveysheet.Surveysheetid = t.Surveysheetid
	CSurveysheet.Cnr = t.Cnr
	CSurveysheet.Surveyfinishedcount = subject.Surveyfinishedcount + 1
	if CSurveysheet.Surveyfinishedcount >= subject.Surveysamplecount {
		CSurveysheet.Status = 2
	} else {
		CSurveysheet.Status = 1
	}
	_, err = session.Table("c_surveysheet").Cols("status", "surveyfinishedcount").ID(core.PK{subject.Cid, subject.Cnr, subject.Surveysheetid}).Update(CSurveysheet)
	if err != nil {
		session.Rollback()
		return err
	}
	//添加答题
	Survey := new(CSurveysheetResult)
	Survey.Cid = t.Cid
	Survey.Cnr = t.Cnr
	Survey.Surveysheetid = t.Surveysheetid
	Survey.Buildingid = t.Buildingid
	Survey.Unit = t.Unit
	Survey.Room = t.Room
	Survey.Surveynr = utils.ValueToInt(id, 0)
	Survey.Surveyperson = subject.Surveyperson
	Survey.Surveysamplecount = subject.Surveysamplecount
	Survey.Surveyfinishedcount = subject.Surveyfinishedcount
	Survey.Remark = t.Remark
	Survey.Signature = t.Signature
	Survey.Planstartdate = t.Planstartdate
	Survey.Planendate = t.Planendate
	Survey.Actenddate = utils.TimeFormat(time.Now(), "yyyy-MM-dd HH:mm:ss")
	Survey.Status = t.Status
	Survey.Lastmodifyby = t.Lastmodifyby
	Survey.Createtime = t.Createtime
	Survey.Lastmodifytime = t.Lastmodifytime
	Survey.Sex = t.Sex
	Survey.PersonType = t.PersonType
	Survey.Age = t.Age
	Survey.LiveTime = t.LiveTime
	Survey.PersonName = t.PersonName
	//Survey.SurveyDate = t.SurveyDate
	//Survey.SurveyHour = t.SurveyHour
	_, err = session.Table("c_surveysheet_result").Insert(Survey)
	if err != nil {
		glog.InfoExt("问题调研", "添加题目 err1 is :", err)
		session.Rollback()
		return err
	}

	//记录答案
	for _, v := range t.Valst {
		//查询问题类型
		subjectlst := new(CSurveysheetSubjectlst)
		ok, err := session.Table("c_surveysheet_subjectlst").Where("cid = ? and subjectid = ?", t.Cid, v.Subjectid).Get(subjectlst)
		if err != nil {
			session.Rollback()
			return err
		}
		if !ok {
			session.Rollback()
			return errors.New("没有该试题！")
		}
		if subjectlst.Subjecttype == "简述题" {
			subject := new(CSurveysheetResultlst)
			subject.Cid = t.Cid
			subject.Cnr = t.Cnr
			subject.Subjectid = v.Subjectid
			subject.Surveynr = utils.ValueToInt(id, 0)
			subject.Surveysheetid = t.Surveysheetid
			subject.Status = v.Status
			subject.SelectedOptions = ""
			subject.Assessmentvalue = 0
			subject.Remark = v.Remark
			subject.Optiontype = v.Optiontype
			subject.Optioninput = v.Optioninput
			subject.Lastmodifyby = t.Lastmodifyby
			subject.Createtime = t.Createtime
			subject.Lastmodifytime = t.Lastmodifytime
			_, err = session.Table("c_surveysheet_resultlst").Insert(subject)
			if err != nil {
				glog.InfoExt("问题调研", "添加问题 err2 is :", err)
				session.Rollback()
				return err
			}
		} else {
			//查询答案分值
			datalist := make([]SubjectOptionlst, 0)
			SelectedOptions := strings.Split(v.SelectedOptions, ",")
			err = engine.Table("subject_optionlst").Where("cid = ? and subjectid = ?", t.Cid, v.Subjectid).In("optionid", SelectedOptions).Find(&datalist)
			for _, vv := range datalist {
				subject := new(CSurveysheetResultlst)
				subject.Cid = t.Cid
				subject.Cnr = t.Cnr
				subject.Subjectid = vv.Subjectid
				subject.Surveynr = utils.ValueToInt(id, 0)
				subject.Surveysheetid = t.Surveysheetid
				subject.Status = v.Status
				subject.SelectedOptions = vv.Optionid
				subject.Assessmentvalue = vv.Optionvalue
				subject.Remark = v.Remark
				subject.Optiontype = vv.Optiontype
				subject.Optioninput = v.Optioninput
				subject.Lastmodifyby = t.Lastmodifyby
				subject.Createtime = t.Createtime
				subject.Lastmodifytime = t.Lastmodifytime
				_, err = session.Table("c_surveysheet_resultlst").Insert(subject)
			}

			if err != nil {
				glog.InfoExt("问题调研", "添加问题 err2 is :", err)
				session.Rollback()
				return err
			}
		}

	}

	err = session.Commit()
	if err != nil {
		return err
	}
	return nil
}

type Satisfaction struct {
	CSurveysheetSubjectlst `xorm:"extends"`
	CSurveysheetOptionlst  `xorm:"extends"`
}

type DataTotal struct {
	SubjectCategoryid string    `json:"subject_categoryid"`
	PerTotal          float64   `json:"per_total"`
	SonData           []SonData `json:"son_data"`
}

//子项
type SonData struct {
	SubjectCategoryid string  `json:"subject_categoryid"`
	SubjectName       string  `json:"subject_name"`
	Asum              int     `json:"asum"`
	APer              float64 `json:"aper"`
	Bsum              int     `json:"bsum"`
	BPer              float64 `json:"bper"`
	Csum              int     `json:"csum"`
	CPer              float64 `json:"cper"`
	Dsum              int     `json:"dsum"`
	DPer              float64 `json:"dper"`
	Esum              int     `json:"esum"`
	EPer              float64 `json:"eper"`
	AllPer            float64 `json:"allper"`
}

type SearchConditions struct {
	AgeSubjectid  string `json:"age_subjectid"`
	AgeOption     string `json:"age_option"`
	SexSubjectid  string `json:"sex_subjectid"`
	SexOption     string `json:"sex_option"`
	LifeSubjectid string `json:"life_subjectid"`
	LifeOption    string `json:"life_option"`
}

/****
*思路：查询所有的题目和答案，然后在内存里做处理
**********/
//满意度统计(单选题)
func (t *CSurveysheetResult) SatisfactionTotal(search SearchConditions) ([]DataTotal, error) {
	total := make([]DataTotal, 0)
	temdatas := make([]DataTotal, 0)
	//联查出类型是整体满意度的所有记录
	engine := db.MasterEngine()
	//查询出统计问题及其答案
	data := make([]Satisfaction, 0)
	err := engine.Table("c_surveysheet_subjectlst").Join("INNER", "c_surveysheet_optionlst", "c_surveysheet_subjectlst.cid = c_surveysheet_optionlst.cid and c_surveysheet_subjectlst.subjectid = c_surveysheet_optionlst.subjectid and c_surveysheet_subjectlst.surveysheetid = c_surveysheet_optionlst.surveysheetid").Where("c_surveysheet_subjectlst.cid = ? and c_surveysheet_subjectlst.surveysheetid = ? and c_surveysheet_subjectlst.subjecttype = ? and  c_surveysheet_subjectlst.subject_categoryid != ?", t.Cid, t.Surveysheetid, "单选题", "基本信息").Desc("c_surveysheet_subjectlst.subject_categoryid").Find(&data)
	if err != nil {
		return total, err
	}
	query := engine.Table("c_surveysheet_resultlst").Where("cid = ? and surveysheetid = ? ", t.Cid, t.Surveysheetid)
	if search.AgeSubjectid != "" && search.AgeOption != "" {
		query = query.And("surveynr in (SELECT surveynr FROM `c_surveysheet_resultlst` WHERE (cid = ? and surveysheetid = ? and subjectid = ? and selected_options = ?))", t.Cid, t.Surveysheetid, search.AgeSubjectid, search.AgeOption)
	}
	if search.SexSubjectid != "" && search.SexOption != "" {
		query = query.And("surveynr in (SELECT surveynr FROM `c_surveysheet_resultlst` WHERE (cid = ? and surveysheetid = ? and subjectid = ? and selected_options = ?))", t.Cid, t.Surveysheetid, search.SexSubjectid, search.SexOption)
	}
	if search.LifeSubjectid != "" && search.LifeOption != "" {
		query = query.And("surveynr in (SELECT surveynr FROM `c_surveysheet_resultlst` WHERE (cid = ? and surveysheetid = ? and subjectid = ? and selected_options = ?))", t.Cid, t.Surveysheetid, search.LifeSubjectid, search.LifeOption)
	}
	results := make([]CSurveysheetResultlst, 0)
	err = query.Find(&results)
	if err != nil {
		return total, err
	}
	dataMap := make(map[string]int)
	//统计出个数
	for _, v := range results {
		key := v.Subjectid + "_" + v.SelectedOptions
		val, ok := dataMap[key]
		if ok {
			val = val + 1
			dataMap[key] = val
		} else {
			val = 1
			dataMap[key] = val
		}
	}
	result := make([]SonData, 0)
	tem := make(map[string][]Satisfaction)
	//计算题目和答案对应的百分比
	for _, v := range data {
		key := v.CSurveysheetSubjectlst.Subjectid
		tem[key] = append(tem[key], v)
	}
	for _, v := range tem {
		one := SonData{}
		all := 0
		for _, vv := range v {
			key := vv.CSurveysheetSubjectlst.Subjectid + "_" + vv.Optionid
			one.SubjectCategoryid = vv.CSurveysheetSubjectlst.SubjectCategoryid
			one.SubjectName = vv.CSurveysheetSubjectlst.Subjecttext
			switch vv.Optionid {
			case "A":
				val, ok := dataMap[key]
				if ok {
					one.Asum = val
				}
				all += val
			case "B":
				val, ok := dataMap[key]
				if ok {
					one.Bsum = val
				}
				all += val
			case "C":
				val, ok := dataMap[key]
				if ok {
					one.Csum = val
				}
				all += val
			case "D":
				val, ok := dataMap[key]
				if ok {
					one.Dsum = val
				}
				all += val
			case "E":
				val, ok := dataMap[key]
				if ok {
					one.Esum = val
				}
				all += val
			}
		}

		//满意度平均分 = (a×5+b×4+c×3+d×2+e×1) ÷ （a+b+c+d+e）
		//满意度 = （满意度平均分÷5 ）× 100%
		if all > 0 {
			one.APer = utils.ValueToFloat(one.Asum, 0.0) / utils.ValueToFloat(all, 0.0)
			one.BPer = utils.ValueToFloat(one.Bsum, 0.0) / utils.ValueToFloat(all, 0.0)
			one.CPer = utils.ValueToFloat(one.Csum, 0.0) / utils.ValueToFloat(all, 0.0)
			one.DPer = utils.ValueToFloat(one.Dsum, 0.0) / utils.ValueToFloat(all, 0.0)
			one.EPer = utils.ValueToFloat(one.Esum, 0.0) / utils.ValueToFloat(all, 0.0)
			AllPer := (one.Asum*5 + one.Bsum*4 + one.Csum*3 + one.Dsum*2 + one.Esum*1)
			one.AllPer = utils.ValueToFloat(AllPer, 0.0) / utils.ValueToFloat(all, 0.0) / utils.ValueToFloat(5, 0.0)
		} else {
			one.APer = 0
			one.BPer = 0
			one.CPer = 0
			one.DPer = 0
			one.EPer = 0
			one.AllPer = 0
		}
		result = append(result, one)
	}
	temdata := make(map[string][]SonData)
	//计算题目和答案对应的百分比
	for _, v := range result {
		key := v.SubjectCategoryid
		temdata[key] = append(temdata[key], v)
	}
	//排序
	Categoryids := []string{}
	for k, v := range temdata {
		one := DataTotal{}
		one.SubjectCategoryid = k
		l := 0
		s := 0.0
		for _, vv := range v {
			l++
			s += vv.AllPer
			one.SonData = append(one.SonData, vv)
		}
		if l > 0 {
			one.PerTotal = utils.ValueToFloat(s, 0.0) / utils.ValueToFloat(l, 0.0)
		} else {
			one.PerTotal = 0
		}
		total = append(total, one)
	}

	temdatastr := make(map[string]DataTotal)
	//计算题目和答案对应的百分比
	for _, v := range total {
		key := v.SubjectCategoryid
		Categoryids = append(Categoryids, key)
		temdatastr[key] = v

	}
	//排序
	sort.Strings(Categoryids)
	for _, key := range Categoryids {
		temdatas = append(temdatas, temdatastr[key])
	}
	return temdatas, nil
}

//多选题
type SubjectsData struct {
	SubjectName  string         `json:"subject_name"`
	SubjectCount int            `json:"subject_count"`
	MultipleData []MultipleData `json:"multiple_data"`
}

type MultipleData struct {
	SubjectName string  `json:"subject_name"`
	OptionName  string  `json:"option_name"`
	Sum         int     `json:"sum"`
	Per         float64 `json:"per"`
}

//满意度统计(多选题)
func (t *CSurveysheetResult) SatisfactionMultipleTotal(search SearchConditions) ([]SubjectsData, error) {
	total := make([]SubjectsData, 0)
	engine := db.MasterEngine()
	//联查出类型是整体满意度的所有记录
	data := make([]Satisfaction, 0)
	err := engine.Table("c_surveysheet_subjectlst").Join("INNER", "c_surveysheet_optionlst", "c_surveysheet_subjectlst.cid = c_surveysheet_optionlst.cid and c_surveysheet_subjectlst.subjectid = c_surveysheet_optionlst.subjectid and c_surveysheet_subjectlst.surveysheetid = c_surveysheet_optionlst.surveysheetid").Where("c_surveysheet_subjectlst.cid = ? and c_surveysheet_subjectlst.surveysheetid = ? and c_surveysheet_subjectlst.subjecttype = ?", t.Cid, t.Surveysheetid, "多选题").Find(&data)
	if err != nil {
		return total, err
	}
	//查询出统计问题及其答案
	query := engine.Table("c_surveysheet_resultlst").Where("cid = ? and surveysheetid = ? ", t.Cid, t.Surveysheetid)
	if search.AgeSubjectid != "" && search.AgeOption != "" {
		query = query.And("surveynr in (SELECT surveynr FROM `c_surveysheet_resultlst` WHERE (cid = ? and surveysheetid = ? and subjectid = ? and selected_options = ?))", t.Cid, t.Surveysheetid, search.AgeSubjectid, search.AgeOption)
	}
	if search.SexSubjectid != "" && search.SexOption != "" {
		query = query.And("surveynr in (SELECT surveynr FROM `c_surveysheet_resultlst` WHERE (cid = ? and surveysheetid = ? and subjectid = ? and selected_options = ?))", t.Cid, t.Surveysheetid, search.SexSubjectid, search.SexOption)
	}
	if search.LifeSubjectid != "" && search.LifeOption != "" {
		query = query.And("surveynr in (SELECT surveynr FROM `c_surveysheet_resultlst` WHERE (cid = ? and surveysheetid = ? and subjectid = ? and selected_options = ?))", t.Cid, t.Surveysheetid, search.LifeSubjectid, search.LifeOption)
	}
	results := make([]CSurveysheetResultlst, 0)
	err = query.Find(&results)
	if err != nil {
		return total, err
	}
	dataMap := make(map[string]int)
	//统计出个数
	for _, v := range results {
		key := v.Subjectid + "_" + v.SelectedOptions
		val, ok := dataMap[key]
		if ok {
			val = val + 1
			dataMap[key] = val
		} else {
			val = 1
			dataMap[key] = val
		}
	}
	Categoryids := []string{}
	temdatas := make(map[string][]Satisfaction)
	tem := make(map[string][]Satisfaction)
	//计算题目和答案对应的百分比
	for _, v := range data {
		key := v.CSurveysheetSubjectlst.Subjecttext
		temdatas[key] = append(temdatas[key], v)
		Categoryids = append(Categoryids, key)
	}
	//排序
	sort.Strings(Categoryids)
	for _, key := range Categoryids {
		tem[key] = temdatas[key]
	}
	for k, v := range tem {
		result := make([]MultipleData, 0)
		subject := SubjectsData{}
		subject.SubjectName = k
		all := 0
		for _, vv := range v {
			one := MultipleData{}
			key := vv.CSurveysheetSubjectlst.Subjectid + "_" + vv.Optionid
			one.SubjectName = vv.CSurveysheetSubjectlst.Subjecttext
			one.OptionName = vv.CSurveysheetOptionlst.Optiontext
			val, ok := dataMap[key]
			if ok {
				one.Sum = val
				all += val
			}
			result = append(result, one)
		}
		subject.SubjectCount = all
		subject.MultipleData = result
		total = append(total, subject)
	}
	//计算题目和答案对应的百分比
	for k, _ := range total {
		for kk, _ := range total[k].MultipleData {
			total[k].MultipleData[kk].Per = utils.ValueToFloat(total[k].MultipleData[kk].Sum, 0.0) / utils.ValueToFloat(total[k].SubjectCount, 0.0)
		}
	}
	return total, nil
}

type SingleSubjectInfo struct {
	SubjectName string `json:"subjectName"`
	Count       int    `json:"count"`
}

type JoinResult struct {
	CSurveysheetResult     `xorm:"extends"`
	CSurveysheetResultlst  `xorm:"extends"`
	CSurveysheetSubjectlst `xorm:"extends"`
	Buildingtab            `xorm:"extends"`
}

type CourtJoinResult struct {
	Cid               int    `json:"cid"`
	Buildingid        string `json:"buildingid"`
	Descr             string `json:"descr"`
	SelectedOptions   string `json:"selected_options"`
	Actenddate        string `json:"actenddate"`
	Totalhouseholds   int    `json:"totalhouseholds"`
	SubjectCategoryid string `json:"subject_categoryid"`
}

// 满意度统计
func (c *CSurveysheetResult) Statistic(selectType string,year string,unit string) (result StatisticData, err error) {
	engine := db.MasterEngine()
	engine.ShowSQL(true)
	query := engine.Table("c_surveysheet_resultlst").Join("LEFT", "c_surveysheet_result", "c_surveysheet_resultlst.cid = c_surveysheet_result.cid and c_surveysheet_resultlst.surveynr = c_surveysheet_result.surveynr")
	query = query.Join("LEFT", "c_surveysheet_subjectlst", "c_surveysheet_resultlst.cid = c_surveysheet_subjectlst.cid and c_surveysheet_resultlst.subjectid = c_surveysheet_subjectlst.subjectid")
	query = query.Join("LEFT", "buildingtab", "c_surveysheet_result.cid = buildingtab.cid and c_surveysheet_result.buildingid = buildingtab.buildingid")
	query = query.Where("c_surveysheet_resultlst.cid = ? and c_surveysheet_resultlst.assessmentvalue != 0", c.Cid)
	joinResultLi := make([]JoinResult, 0)
	err = query.Find(&joinResultLi)
	if err != nil {
		return result, err
	}
	cateMap := make(map[string][]JoinResult)
	for _, joinResult := range joinResultLi {
		datetimeStr := joinResult.Actenddate
		date, err := time.Parse("2006-01-02 15:04:05", datetimeStr)
		if err != nil {
			return result, err
		}
		yearstr := date.Year()
		if utils.ValueToInt(year,0) != yearstr{
			continue
		}
		month := int(date.Month())
		if unit == "top"{
			if month >6{
				continue
			}
		}else if unit == "bottom"{
			if month <= 6{
				continue
			}
		}else {

		}
		cate := joinResult.CSurveysheetSubjectlst.SubjectCategoryid
		if cate == "整体满意程度" || cate == ""{
			continue
		}
		cateMap[cate] = append(cateMap[cate], joinResult)
	}
	cateStatisticDataLi := make([]CateStatisticData, 0)
	for cate, joinResultLi := range cateMap {
		var cateStatisticData CateStatisticData
		cateStatisticData.Cate = cate
		cateStatisticData.SubjectStaticDataLi = make([]SubjectStaticData, 0)
		tempSubjectMap := make(map[string]SubjectStaticData)
		for _, joinResult := range joinResultLi {
			cateStatisticData.Count++
			var tempSubjectStatisticData SubjectStaticData

			subjectId := joinResult.CSurveysheetResultlst.Subjectid
			_, exist := tempSubjectMap[subjectId]
			if !exist {
				tempSubjectMap[subjectId] = SubjectStaticData{}
				tempSubjectStatisticData = tempSubjectMap[subjectId]
				tempSubjectStatisticData.CateId = cate
				tempSubjectStatisticData.Pos = joinResult.CSurveysheetSubjectlst.Pos
			} else {
				tempSubjectStatisticData = tempSubjectMap[subjectId]
			}
			tempSubjectStatisticData.SubjectName = joinResult.CSurveysheetSubjectlst.Subjecttext
			switch joinResult.CSurveysheetResultlst.SelectedOptions {
			case "A":
				tempSubjectStatisticData.ASum++
				cateStatisticData.ASum++
			case "B":
				tempSubjectStatisticData.BSum++
				cateStatisticData.BSum++
			case "C":
				tempSubjectStatisticData.CSum++
				cateStatisticData.CSum++
			case "D":
				tempSubjectStatisticData.DSum++
				cateStatisticData.DSum++
			case "E":
				tempSubjectStatisticData.ESum++
				cateStatisticData.ESum++
			}
			tempSubjectStatisticData.Count++
			tempSubjectMap[subjectId] = tempSubjectStatisticData
		}
		for _, data := range tempSubjectMap {
			if data.Count == 0 {
				data.APercent = 0
				data.BPercent = 0
				data.CPercent = 0
				data.DPercent = 0
				data.EPercent = 0
			} else {
				data.APercent = utils.Decimal(float64(data.ASum)/float64(data.Count), 3)
				data.BPercent = utils.Decimal(float64(data.BSum)/float64(data.Count), 3)
				data.CPercent = utils.Decimal(float64(data.CSum)/float64(data.Count), 3)
				data.DPercent = utils.Decimal(float64(data.DSum)/float64(data.Count), 3)
				data.EPercent = utils.Decimal(float64(data.ESum)/float64(data.Count), 3)
			}
			cateStatisticData.SubjectStaticDataLi = append(cateStatisticData.SubjectStaticDataLi, data)
		}
		cateStatisticDataLi = append(cateStatisticDataLi, cateStatisticData)
	}
	var totalData TotalStatisticData
	timeStatisticDataLi := make([]TimeStatisticData, 0)
	recentYearLi := make([]RecentYear, 0)

	tempYearMap := make(map[string]RecentYear)
	tempTimeMap := make(map[string]TimeStatisticData)
	tempYearOutMap := make(map[string]YearStatisticData)
	yearStatisticDataLi := make([]YearStatisticData, 0)
	for _, joinResult := range joinResultLi {
		datetimeStr := joinResult.CSurveysheetResult.Actenddate
		date, err := time.Parse("2006-01-02 15:04:05", datetimeStr)
		if err != nil {
			return result, err
		}
		yearstr := date.Year()
		if utils.ValueToInt(year,0) != yearstr{
			continue
		}
		month := int(date.Month())
		var innerTimeType string
		if month > 6 {
			innerTimeType = "年末"
		} else {
			innerTimeType = "年中"
		}

		if unit == "top"{
			innerTimeType = "年末"
			if month >6{
				continue
			}
		}else if unit == "bottom"{
			if month <= 6{
				continue
			}
		}else {

		}
		if joinResult.CSurveysheetSubjectlst.SubjectCategoryid == "" {
			continue
		}

		descr := fmt.Sprintf("%d%s", yearstr, innerTimeType)
		if joinResult.CSurveysheetSubjectlst.SubjectCategoryid != "整体满意程度" {
			var yearData YearStatisticData
			var cateData SingleCateData
			_, exist := tempYearOutMap[descr]
			if exist {
				yearData = tempYearOutMap[descr]
			} else {
				tempYearOutMap[descr] = YearStatisticData{}
				yearData = tempYearOutMap[descr]
				yearData.TimeDescr = descr
				yearData.Year = yearstr
				yearData.Type = innerTimeType
				yearData.CateData = make([]SingleCateData, 0)
				yearData.CateMap = make(map[string]SingleCateData)
			}
			cateId := joinResult.CSurveysheetSubjectlst.SubjectCategoryid
			_, exist = yearData.CateMap[cateId]
			if exist {
				cateData = yearData.CateMap[cateId]
			} else {
				yearData.CateMap[cateId] = SingleCateData{}
				cateData = yearData.CateMap[cateId]
				cateData.CateId = cateId
			}
			switch joinResult.CSurveysheetResultlst.SelectedOptions {
			case "A":
				cateData.ASum++
			case "B":
				cateData.BSum++
			case "C":
				cateData.CSum++
			case "D":
				cateData.DSum++
			case "E":
				cateData.ESum++
			}
			cateData.Count++
			yearData.CateMap[cateId] = cateData
			tempYearOutMap[descr] = yearData
			continue
		}

		// 处理 按年份的统计数据
		{
			var recentYear RecentYear
			_, exist := tempYearMap[descr]
			if exist {
				recentYear = tempYearMap[descr]
			} else {
				tempYearMap[descr] = RecentYear{}
				recentYear = tempYearMap[descr]
				recentYear.TimeDescr = descr
				recentYear.Year = yearstr
				if innerTimeType == "年中" {
					recentYear.Type = 0
				} else {
					recentYear.Type = 1
				}
			}
			recentYear.Count++
			switch joinResult.CSurveysheetResultlst.SelectedOptions {
			case "A":
				recentYear.ASum++
			case "B":
				recentYear.BSum++
			case "C":
				recentYear.CSum++
			case "D":
				recentYear.DSum++
			case "E":
				recentYear.ESum++
			}
			tempYearMap[descr] = recentYear
		}
		// 处理 按年份 按居住时间的统计数据
		{
			var timeData TimeStatisticData
			_, exist := tempTimeMap[descr]
			if exist {
				timeData = tempTimeMap[descr]
			} else {
				tempTimeMap[descr] = TimeStatisticData{}
				timeData = tempTimeMap[descr]
				timeData.TimeDescr = descr
				timeData.Year = yearstr
				if innerTimeType == "年中" {
					timeData.Type = 0
				} else {
					timeData.Type = 1
				}
			}
			switch joinResult.CSurveysheetResult.LiveTime {
			case 1:
				timeData.BreakingInPeriodCount++
				switch joinResult.CSurveysheetResultlst.SelectedOptions {
				case "A":
					timeData.BreakingInPeriodASum++
				case "B":
					timeData.BreakingInPeriodBSum++
				case "C":
					timeData.BreakingInPeriodCSum++
				case "D":
					timeData.BreakingInPeriodDSum++
				case "E":
					timeData.BreakingInPeriodESum++
				}
			case 2:
				timeData.BreakingInPeriodCount++
				switch joinResult.CSurveysheetResultlst.SelectedOptions {
				case "A":
					timeData.BreakingInPeriodASum++
				case "B":
					timeData.BreakingInPeriodBSum++
				case "C":
					timeData.BreakingInPeriodCSum++
				case "D":
					timeData.BreakingInPeriodDSum++
				case "E":
					timeData.BreakingInPeriodESum++
				}
			case 3:
				timeData.StablePeriodCount++
				switch joinResult.CSurveysheetResultlst.SelectedOptions {
				case "A":
					timeData.StablePeriodASum++
				case "B":
					timeData.StablePeriodBSum++
				case "C":
					timeData.StablePeriodCSum++
				case "D":
					timeData.StablePeriodDSum++
				case "E":
					timeData.StablePeriodESum++
				}
			case 4:
				timeData.AcquaintanceCount++
				switch joinResult.CSurveysheetResultlst.SelectedOptions {
				case "A":
					timeData.AcquaintanceASum++
				case "B":
					timeData.AcquaintanceBSum++
				case "C":
					timeData.AcquaintanceCSum++
				case "D":
					timeData.AcquaintanceDSum++
				case "E":
					timeData.AcquaintanceESum++
				}
			}
			timeData.Count++
			tempTimeMap[descr] = timeData

		}
		// 处理 整体满意度
		{
			totalData.Count++
			switch joinResult.CSurveysheetResultlst.SelectedOptions {
			case "A":
				totalData.ASum++
			case "B":
				totalData.BSum++
			case "C":
				totalData.CSum++
			case "D":
				totalData.DSum++
			case "E":
				totalData.ESum++
			}
		}

	}
	for _, data := range tempYearOutMap {
		for _, singleData := range data.CateMap {
			data.CateData = append(data.CateData, singleData)
		}
		yearStatisticDataLi = append(yearStatisticDataLi, data)
	}

	if totalData.Count == 0 {
		totalData.APercent = 0
		totalData.BPercent = 0
		totalData.CPercent = 0
		totalData.DPercent = 0
		totalData.EPercent = 0
	} else {
		totalData.APercent = utils.Decimal(float64(totalData.ASum)/float64(totalData.Count), 3)
		totalData.BPercent = utils.Decimal(float64(totalData.BSum)/float64(totalData.Count), 3)
		totalData.CPercent = utils.Decimal(float64(totalData.CSum)/float64(totalData.Count), 3)
		totalData.DPercent = utils.Decimal(float64(totalData.DSum)/float64(totalData.Count), 3)
		totalData.EPercent = utils.Decimal(float64(totalData.ESum)/float64(totalData.Count), 3)
	}

	for _, data := range tempYearMap {
		recentYearLi = append(recentYearLi, data)
	}
	for _, data := range tempTimeMap {
		timeStatisticDataLi = append(timeStatisticDataLi, data)
	}
	var buildingData BuildingData
	buildingData.BuildingStaticDataLi = make([]BuildingStaticData, 0)
	tempBuildMap := make(map[string]BuildingStaticData)
	buildMap := make(map[string][]JoinResult, 0)
	for _, joinResult := range joinResultLi {
		datetimeStr := joinResult.Actenddate
		date, err := time.Parse("2006-01-02 15:04:05", datetimeStr)
		if err != nil {
			return result, err
		}
		yearstr := date.Year()
		if utils.ValueToInt(year,0) != yearstr{
			continue
		}
		month := int(date.Month())
		if unit == "top"{
			if month >6{
				continue
			}
		}else if unit == "bottom"{
			if month <= 6{
				continue
			}
		}else {

		}
		buildingDescr := joinResult.Buildingtab.Descr
		if buildingDescr == "" {
			buildingDescr = "未知"
		}
		{
			_, exist := buildMap[buildingDescr]
			if exist {
				buildMap[buildingDescr] = append(buildMap[buildingDescr], joinResult)
			} else {
				buildMap[buildingDescr] = make([]JoinResult, 0)
				buildMap[buildingDescr] = append(buildMap[buildingDescr], joinResult)
			}
		}
		{
			if joinResult.CSurveysheetSubjectlst.SubjectCategoryid == "整体满意程度" {
				var tempBuildData BuildingStaticData
				_, exist := tempBuildMap[buildingDescr]
				if exist {
					tempBuildData = tempBuildMap[buildingDescr]
				} else {
					tempBuildMap[buildingDescr] = BuildingStaticData{}
					tempBuildData = tempBuildMap[buildingDescr]
					tempBuildData.BuildingRooms = joinResult.Buildingtab.Totalhouseholds
					buildingData.Rooms += tempBuildData.BuildingRooms
					tempBuildData.BuildingDescr = joinResult.Buildingtab.Descr
					if tempBuildData.BuildingDescr == "" {
						tempBuildData.BuildingDescr = "未知"
					}
					tempBuildData.BuildingId = joinResult.Buildingtab.Buildingid
				}
				switch joinResult.CSurveysheetResultlst.SelectedOptions {
				case "A":
					tempBuildData.ASum++
					buildingData.ASum++
				case "B":
					tempBuildData.BSum++
					buildingData.BSum++
				case "C":
					tempBuildData.CSum++
					buildingData.CSum++
				case "D":
					tempBuildData.DSum++
					buildingData.DSum++
				case "E":
					tempBuildData.ESum++
					buildingData.ESum++
				}
				tempBuildData.BuildingSamples++
				buildingData.Count++
				tempBuildMap[buildingDescr] = tempBuildData
			}

		}

	}
	buildingCateLi := make([]BuildingCate, 0)
	for buildingDescr, li := range buildMap {
		var buildingCate BuildingCate
		buildingCate.BuildingDescr = buildingDescr
		buildingCate.CateData = make([]SingleCateData, 0)
		cateBuildMap := make(map[string]SingleCateData)
		for _, joinResult := range li {
			if joinResult.CSurveysheetSubjectlst.SubjectCategoryid == "整体满意程度" {
				buildingCate.Samples++
			}
			cateId := joinResult.CSurveysheetSubjectlst.SubjectCategoryid
			var cateData SingleCateData
			_, exist := cateBuildMap[cateId]
			if exist {
				cateData = cateBuildMap[cateId]
			} else {
				cateBuildMap[cateId] = SingleCateData{}
				cateData.CateId = cateId
				cateData.BuildingDescr = buildingDescr
			}
			cateData.Count++
			switch joinResult.CSurveysheetResultlst.SelectedOptions {
			case "A":
				cateData.ASum++
			case "B":
				cateData.BSum++
			case "C":
				cateData.CSum++
			case "D":
				cateData.DSum++
			case "E":
				cateData.ESum++
			}
			cateBuildMap[cateId] = cateData
		}
		for _, data := range cateBuildMap {
			buildingCate.CateData = append(buildingCate.CateData, data)
		}
		buildingCateLi = append(buildingCateLi, buildingCate)
	}

	for _, data := range tempBuildMap {
		if data.BuildingRooms == 0 {
			data.SampleBuildingPercent = 0
		} else {
			data.SampleBuildingPercent = utils.Decimal(float64(data.BuildingSamples)/float64(data.BuildingRooms), 3)
		}
		buildingData.BuildingStaticDataLi = append(buildingData.BuildingStaticDataLi, data)
	}

	// recentYear 排序
	li := make([]ICompare, 0, len(recentYearLi))
	for _, recentYear := range recentYearLi {
		li = append(li, recentYear)
	}
	li = BubbleSort(li)
	result.RecentYears = make([]RecentYear, 0, len(li))
	for _, data := range li {
		result.RecentYears = append(result.RecentYears, data.(RecentYear))
	}

	// timeStatisticData 排序
	li = make([]ICompare, 0, len(timeStatisticDataLi))
	for _, timeStatisticData := range timeStatisticDataLi {
		li = append(li, timeStatisticData)
	}
	li = BubbleSort(li)
	result.Time = make([]TimeStatisticData, 0, len(timeStatisticDataLi))
	for _, data := range li {
		result.Time = append(result.Time, data.(TimeStatisticData))
	}

	// cateSatisticData排序
	li = make([]ICompare, 0, len(cateStatisticDataLi))
	for _, cateStatisticData := range cateStatisticDataLi {
		li = append(li, cateStatisticData)
	}
	li = BubbleSort(li)
	result.Cate = make([]CateStatisticData, 0, len(cateStatisticDataLi))
	for _, data := range li {
		result.Cate = append(result.Cate, data.(CateStatisticData))
	}
	// building data 排序
	buildingLi := buildingData.BuildingStaticDataLi
	li = make([]ICompare, 0, len(buildingLi))
	for _, building := range buildingLi {
		li = append(li, building)
	}
	li = BubbleSort(li)
	buildingData.BuildingStaticDataLi = make([]BuildingStaticData, 0, len(buildingLi))
	for _, data := range li {
		buildingData.BuildingStaticDataLi = append(buildingData.BuildingStaticDataLi, data.(BuildingStaticData))
	}
	result.Building = buildingData
	// cateSatisticData排序
	li = make([]ICompare, 0, len(yearStatisticDataLi))
	for _, yearData := range yearStatisticDataLi {
		li = append(li, yearData)
	}
	li = BubbleSort(li)
	result.YearCate = make([]YearStatisticData, 0, len(yearStatisticDataLi))
	for _, data := range li {
		result.YearCate = append(result.YearCate, data.(YearStatisticData))
	}
	// building cate 排序 cate
	for index, buildingCateData := range buildingCateLi {
		tempDataLi := buildingCateData.CateData
		li = make([]ICompare, 0, len(buildingCateData.CateData))
		buildingCateData.CateData = make([]SingleCateData, 0, len(buildingCateData.CateData))
		for _, data := range tempDataLi {
			li = append(li, data)
		}
		li = BubbleSort(li)
		for _, data := range li {
			buildingCateData.CateData = append(buildingCateData.CateData, data.(SingleCateData))
		}
		buildingCateLi[index] = buildingCateData
	}
	//str, _ := json.MarshalIndent(buildingCateLi, "", "  ")
	//fmt.Println(string(str))
	// 将各个分类统计的值赋值到result
	result.Total = totalData
	result.PicTotal = PicTotal{}
	result.BuildingCate = buildingCateLi

	if selectType == "ratio" {
		// 处理整体满意度的表格
		result.Total.SatisfactionPercent = utils.Decimal(float64(result.Total.ASum+result.Total.BSum)/float64(result.Total.Count), 3)
		// 处理饼图数据
		result.PicTotal.Row = []string{"非常满意", "满意", "一般", "不满意", "非常不满意"}
		result.PicTotal.ValueLi = []float64{totalData.APercent, totalData.BPercent, totalData.CPercent, totalData.DPercent, totalData.EPercent}
		// 处理每个年不同时期的满意度调查
		for index, recentYear := range result.RecentYears {
			if recentYear.Count == 0 {
				recentYear.SatisfactionPercent = 0
			} else {
				recentYear.SatisfactionPercent = utils.Decimal(float64(recentYear.ASum+recentYear.BSum)/float64(recentYear.Count), 3)
			}
			result.RecentYears[index] = recentYear
		}
		// 处理每个年不同时期 不同居住时间的满意度调查
		for index, data := range result.Time {
			if data.BreakingInPeriodCount == 0 {
				data.BreakingInPeriodPercent = 0
			} else {
				data.BreakingInPeriodPercent = utils.Decimal(float64(data.BreakingInPeriodASum+data.BreakingInPeriodBSum)/float64(data.BreakingInPeriodCount), 3)
			}
			if data.StablePeriodCount == 0 {
				data.StablePeriodPercent = 0
			} else {
				data.StablePeriodPercent = utils.Decimal(float64(data.StablePeriodASum+data.StablePeriodBSum)/float64(data.StablePeriodCount), 3)
			}
			if data.AcquaintanceCount == 0 {
				data.AcquaintancePercent = 0
			} else {
				data.AcquaintancePercent = utils.Decimal(float64(data.AcquaintanceASum+data.AcquaintanceBSum)/float64(data.AcquaintanceCount), 3)
			}
			result.Time[index] = data
		}
		// 处理不同subject的满意度调差
		for index, data := range result.Cate {
			for innerIndex, subjectData := range data.SubjectStaticDataLi {
				if subjectData.Count == 0 {
					subjectData.SatisfactionPercent = 0
				} else {
					subjectData.SatisfactionPercent = utils.Decimal(subjectData.APercent + subjectData.BPercent, 3)
				}
				data.SubjectStaticDataLi[innerIndex] = subjectData
			}
			if data.Count == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum)/float64(data.Count), 3)
			}
			result.Cate[index] = data
		}
		// 处理楼栋的满意度统计
		if result.Building.Count == 0 {
			result.Building.SatisfactionPercent = 0
		} else {
			result.Building.SatisfactionPercent = utils.Decimal(float64(result.Building.ASum+result.Building.BSum)/float64(result.Building.Count), 3)
		}
		if result.Building.Rooms == 0 {
			result.Building.SampleBuildingPercent = 0
		} else {
			result.Building.SampleBuildingPercent = utils.Decimal(float64(result.Building.Count)/float64(result.Building.Rooms), 3)
		}
		result.Building.SampleAllPercent = 1
		for index, data := range result.Building.BuildingStaticDataLi {
			if data.BuildingSamples == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum)/float64(data.BuildingSamples), 3)
			}
			data.DifferencePercent = utils.Decimal(data.SatisfactionPercent-result.Building.SatisfactionPercent, 3)
			if result.Building.Count == 0 {
				data.SampleAllPercent = 0
			} else {
				data.SampleAllPercent = utils.Decimal(float64(data.BuildingSamples)/float64(result.Building.Count), 3)
			}
			result.Building.BuildingStaticDataLi[index] = data
		}
		// 处理楼栋分类满意度统计
		for index, buildingCate := range result.BuildingCate {
			for innerIndex, cate := range buildingCate.CateData {
				if cate.Count == 0 {
					cate.SatisfactionPercent = 0
				} else {
					cate.SatisfactionPercent = utils.Decimal(float64(cate.ASum+cate.BSum)/float64(cate.Count), 3)
				}
				buildingCate.CateData[innerIndex] = cate
			}
			result.BuildingCate[index] = buildingCate
		}
		// 处理年份分类满意度
		for index, yearData := range result.YearCate {
			for innerIndex, cateData := range yearData.CateData {
				if cateData.Count == 0 {
					cateData.SatisfactionPercent = 0
				} else {
					cateData.SatisfactionPercent = utils.Decimal(float64(cateData.ASum+cateData.BSum)/float64(cateData.Count), 3)

				}
				yearData.CateData[innerIndex] = cateData
			}
			yearData.CateMap = nil
			result.YearCate[index] = yearData
		}
	} else if selectType == "threeratio" {
		/********
		*三分比例法:“3分比例法”统计的是选择“非常满意”、“满意”和“一般”的人数占比,此三项加和作为分母
		**************/
		// 处理整体满意度的表格
		result.Total.SatisfactionPercent = result.Total.APercent + result.Total.BPercent + result.Total.CPercent
		// 处理饼图数据
		result.PicTotal.Row = []string{"非常满意", "满意", "一般", "不满意", "非常不满意"}
		result.PicTotal.ValueLi = []float64{totalData.APercent, totalData.BPercent, totalData.CPercent, totalData.DPercent, totalData.EPercent}
		// 处理每个年不同时期的满意度调查
		for index, recentYear := range result.RecentYears {
			if recentYear.Count == 0 {
				recentYear.SatisfactionPercent = 0
			} else {
				recentYear.SatisfactionPercent = utils.Decimal(float64(recentYear.ASum+recentYear.BSum+recentYear.CSum)/float64(recentYear.Count), 3)
			}
			result.RecentYears[index] = recentYear
		}
		// 处理每个年不同时期 不同居住时间的满意度调查
		for index, data := range result.Time {
			if data.BreakingInPeriodCount == 0 {
				data.BreakingInPeriodPercent = 0
			} else {
				data.BreakingInPeriodPercent = utils.Decimal(float64(data.BreakingInPeriodASum+data.BreakingInPeriodBSum+data.BreakingInPeriodCSum)/float64(data.BreakingInPeriodCount), 3)
			}
			if data.StablePeriodCount == 0 {
				data.StablePeriodPercent = 0
			} else {
				data.StablePeriodPercent = utils.Decimal(float64(data.StablePeriodASum+data.StablePeriodBSum+data.StablePeriodCSum)/float64(data.StablePeriodCount), 3)
			}
			if data.AcquaintanceCount == 0 {
				data.AcquaintancePercent = 0
			} else {
				data.AcquaintancePercent = utils.Decimal(float64(data.AcquaintanceASum+data.AcquaintanceBSum+data.AcquaintanceCSum)/float64(data.AcquaintanceCount), 3)
			}
			result.Time[index] = data
		}
		// 处理不同subject的满意度调差
		for index, data := range result.Cate {
			for innerIndex, subjectData := range data.SubjectStaticDataLi {
				if subjectData.Count == 0 {
					subjectData.SatisfactionPercent = 0
				} else {
					subjectData.SatisfactionPercent = subjectData.APercent + subjectData.BPercent + subjectData.CPercent
				}
				data.SubjectStaticDataLi[innerIndex] = subjectData
			}
			if data.Count == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum+data.CSum)/float64(data.Count), 3)
			}
			result.Cate[index] = data
		}
		// 处理楼栋的满意度统计
		if result.Building.Count == 0 {
			result.Building.SatisfactionPercent = 0
		} else {
			result.Building.SatisfactionPercent = utils.Decimal(float64(result.Building.ASum+result.Building.BSum+result.Building.CSum)/float64(result.Building.Count), 3)
		}
		if result.Building.Rooms == 0 {
			result.Building.SampleBuildingPercent = 0
		} else {
			result.Building.SampleBuildingPercent = utils.Decimal(float64(result.Building.Count)/float64(result.Building.Rooms), 3)
		}
		result.Building.SampleAllPercent = 1
		for index, data := range result.Building.BuildingStaticDataLi {
			if data.BuildingSamples == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum+data.CSum)/float64(data.BuildingSamples), 3)
			}
			data.DifferencePercent = utils.Decimal(data.SatisfactionPercent-result.Building.SatisfactionPercent, 3)
			if result.Building.Count == 0 {
				data.SampleAllPercent = 0
			} else {
				data.SampleAllPercent = utils.Decimal(float64(data.BuildingSamples)/float64(result.Building.Count), 3)
			}
			result.Building.BuildingStaticDataLi[index] = data
		}
		// 处理楼栋分类满意度统计
		for index, buildingCate := range result.BuildingCate {
			for innerIndex, cate := range buildingCate.CateData {
				if cate.Count == 0 {
					cate.SatisfactionPercent = 0
				} else {
					cate.SatisfactionPercent = utils.Decimal(float64(cate.ASum+cate.BSum+cate.CSum)/float64(cate.Count), 3)
				}
				buildingCate.CateData[innerIndex] = cate
			}
			result.BuildingCate[index] = buildingCate
		}
		// 处理年份分类满意度
		for index, yearData := range result.YearCate {
			for innerIndex, cateData := range yearData.CateData {
				if cateData.Count == 0 {
					cateData.SatisfactionPercent = 0
				} else {
					cateData.SatisfactionPercent = utils.Decimal(float64(cateData.ASum+cateData.BSum+cateData.CSum)/float64(cateData.Count), 3)

				}
				yearData.CateData[innerIndex] = cateData
			}
			yearData.CateMap = nil
			result.YearCate[index] = yearData
		}
	} else {
		// 处理整体满意度的表格
		result.Total.SatisfactionPercent = utils.Decimal(float64(totalData.ASum*5+totalData.BSum*4+totalData.CSum*3+totalData.DSum*2+totalData.ESum*1)/float64(5*totalData.Count), 3)
		// 处理饼图数据
		result.PicTotal.Row = []string{"非常满意", "满意", "一般", "不满意", "非常不满意"}
		result.PicTotal.ValueLi = []float64{totalData.APercent, totalData.BPercent, totalData.CPercent, totalData.DPercent, totalData.EPercent}
		// 处理每个年不同时期的满意度调查
		for index, recentYear := range result.RecentYears {
			if recentYear.Count == 0 {
				recentYear.SatisfactionPercent = 0
			} else {
				recentYear.SatisfactionPercent = utils.Decimal(float64(recentYear.ASum*5+recentYear.BSum*4+recentYear.CSum*3+recentYear.DSum*2+recentYear.ESum*1)/float64(5*recentYear.Count), 3)
			}
			result.RecentYears[index] = recentYear
		}
		// 处理每个年不同时期 不同居住时间的满意度调查
		for index, data := range result.Time {
			if data.BreakingInPeriodCount == 0 {
				data.BreakingInPeriodPercent = 0
			} else {
				data.BreakingInPeriodPercent = utils.Decimal(float64(data.BreakingInPeriodASum*5+data.BreakingInPeriodBSum*4+data.BreakingInPeriodCSum*3+data.BreakingInPeriodDSum*2+data.BreakingInPeriodESum*1)/float64(5*data.BreakingInPeriodCount), 3)
			}
			if data.StablePeriodCount == 0 {
				data.StablePeriodPercent = 0
			} else {
				data.StablePeriodPercent = utils.Decimal(float64(data.StablePeriodASum*5+data.StablePeriodBSum*4+data.StablePeriodCSum*3+data.StablePeriodDSum*2+data.StablePeriodESum*1)/float64(5*data.StablePeriodCount), 3)
			}
			if data.AcquaintanceCount == 0 {
				data.AcquaintancePercent = 0
			} else {
				data.AcquaintancePercent = utils.Decimal(float64(data.AcquaintanceASum*5+data.AcquaintanceBSum*4+data.AcquaintanceCSum*3+data.AcquaintanceDSum*2+data.AcquaintanceESum*1)/float64(5*data.AcquaintanceCount), 3)
			}
			result.Time[index] = data
		}
		// 处理不同subject的满意度调差
		for index, data := range result.Cate {
			for innerIndex, subjectData := range data.SubjectStaticDataLi {
				if subjectData.Count == 0 {
					subjectData.SatisfactionPercent = 0
				} else {
					subjectData.SatisfactionPercent = utils.Decimal(float64(subjectData.ASum*5+subjectData.BSum*4+subjectData.CSum*3+subjectData.DSum*2+subjectData.ESum*1)/float64(subjectData.Count*5), 3)
				}
				data.SubjectStaticDataLi[innerIndex] = subjectData
			}
			if data.Count == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum*5+data.BSum*4+data.CSum*3+data.DSum*2+data.ESum*1)/float64(data.Count*5), 3)
			}
			result.Cate[index] = data
		}

		// 处理楼栋的满意度统计
		if result.Building.Count == 0 {
			result.Building.SatisfactionPercent = 0

		} else {
			result.Building.SatisfactionPercent = utils.Decimal(float64(result.Building.ASum*5+result.Building.BSum*4+result.Building.CSum*3+result.Building.BSum*2+result.Building.ESum*1)/float64(result.Building.Count*5), 3)
			result.Building.DifferencePercent = utils.Decimal(1-result.Building.SatisfactionPercent, 3)
		}
		if result.Building.Rooms == 0 {
			result.Building.SampleBuildingPercent = 0
		} else {
			result.Building.SampleBuildingPercent = utils.Decimal(float64(result.Building.Count)/float64(result.Building.Rooms), 3)
		}
		result.Building.SampleAllPercent = 1
		for index, data := range result.Building.BuildingStaticDataLi {

			if data.BuildingSamples == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum*5+data.BSum*4+data.CSum*3+data.DSum*2+data.ESum*1)/float64(data.BuildingSamples*5), 3)
			}
			data.DifferencePercent = utils.Decimal(data.SatisfactionPercent-result.Building.SatisfactionPercent, 3)
			if result.Building.Count == 0 {
				data.SampleAllPercent = 0
			} else {
				data.SampleAllPercent = utils.Decimal(float64(data.BuildingSamples)/float64(result.Building.Count), 3)
			}
			result.Building.BuildingStaticDataLi[index] = data
		}

		// 处理楼栋分类满意度统计
		for index, buildingCate := range result.BuildingCate {
			for innerIndex, cate := range buildingCate.CateData {
				if cate.Count == 0 {
					cate.SatisfactionPercent = 0
				} else {
					cate.SatisfactionPercent = utils.Decimal(float64(cate.ASum*5+cate.BSum*4+cate.CSum*3+cate.DSum*2+cate.ESum)/float64(cate.Count*5), 3)
				}
				buildingCate.CateData[innerIndex] = cate
			}
			result.BuildingCate[index] = buildingCate
		}

		// 处理年份分类满意度
		for index, yearData := range result.YearCate {
			for innerIndex, cateData := range yearData.CateData {
				if cateData.Count == 0 {
					cateData.SatisfactionPercent = 0
				} else {
					cateData.SatisfactionPercent = utils.Decimal(float64(cateData.ASum*5+cateData.BSum*4+cateData.CSum*3+cateData.DSum*2+cateData.ESum)/float64(cateData.Count*5), 3)
				}
				yearData.CateData[innerIndex] = cateData
			}
			yearData.CateMap = nil
			result.YearCate[index] = yearData
		}
	}
	// 处理buildCate 处理成map
	for index, buildCate := range result.BuildingCate {
		buildCate.CateMap = make([]map[string]float64, 0)
		for _, cateData := range buildCate.CateData {
			tempMap := make(map[string]float64, 0)
			tempMap[cateData.CateId] = cateData.SatisfactionPercent
			buildCate.CateMap = append(buildCate.CateMap, tempMap)
		}
		result.BuildingCate[index] = buildCate
	}

	// building cate data 排序
	buildingCateLi = result.BuildingCate
	li = make([]ICompare, 0, len(buildingCateLi))
	for _, buildingCate := range buildingCateLi {
		li = append(li, buildingCate)
	}
	li = BubbleSort(li)
	result.BuildingCate = make([]BuildingCate, 0, len(buildingCateLi))
	for _, data := range li {
		result.BuildingCate = append(result.BuildingCate, data.(BuildingCate))
	}
	return result, nil
}

// 满意度统计
func (c *CSurveysheetResult) AllStatistic(selectType string, cids []string, year string, unit string) (result GatherStatisticData, err error) {
	engine := db.MasterEngine()
	//engine.ShowSQL(true)
	query := engine.Table("c_surveysheet_resultlst").Select("courttab.cid,c_surveysheet_resultlst.selected_options,c_surveysheet_subjectlst.subject_categoryid,courttab.descr,buildingtab.totalhouseholds,buildingtab.Buildingid,c_surveysheet_result.actenddate")
	query = query.Join("LEFT", "c_surveysheet_result", "c_surveysheet_resultlst.cid = c_surveysheet_result.cid and c_surveysheet_resultlst.surveynr = c_surveysheet_result.surveynr")
	query = query.Join("LEFT", "c_surveysheet_subjectlst", "c_surveysheet_resultlst.cid = c_surveysheet_subjectlst.cid and c_surveysheet_resultlst.subjectid = c_surveysheet_subjectlst.subjectid")
	query = query.Join("LEFT", "buildingtab", "c_surveysheet_result.cid = buildingtab.cid and c_surveysheet_result.buildingid = buildingtab.buildingid")
	query = query.Join("LEFT", "courttab", "c_surveysheet_resultlst.cid = courttab.cid")
	query = query.In("c_surveysheet_resultlst.cid", cids).Where("c_surveysheet_resultlst.assessmentvalue != 0")
	joinResultLi := make([]CourtJoinResult, 0)
	err = query.Find(&joinResultLi)
	if err != nil {
		return result, err
	}

	menus := []string{"公共秩序", "维修服务", "清洁绿化", "客服人员", "社区文化", "停车场"}
	//集团单项满意度汇总
	cipsMap := make(map[string][]CourtJoinResult)
	for _, joinResult := range joinResultLi {
		cid := strings.TrimSpace(utils.ValueToString(joinResult.Cid, ""))
		cipsMap[cid] = append(cipsMap[cid], joinResult)
	}
	cidBuildMap := make(map[string]int)
	TotalStatisticData := make([]GatherTotalStatisticData, 0)

	//所有园区的数据
	CidStatisticDatas := make([]CidStatisticData, 0)

	//记录集团
	var oneGroup GatherTotalStatisticData
	oneGroup.CidName = "集团"
	var oneCidGroup CidStatisticData
	oneCidGroup.CidName = "集团"
	groupSubjectMap := make(map[string]SubjectStaticData)
	for cid, cidResultLi := range cipsMap {
		var totalData GatherTotalStatisticData
		var oneCid CidStatisticData
		tempSubjectMap := make(map[string]SubjectStaticData)
		for _, joinResult := range cidResultLi {

			datetimeStr := joinResult.Actenddate
			date, err := time.Parse("2006-01-02 15:04:05", datetimeStr)
			if err != nil {
				return result, err
			}
			yearstr := date.Year()
			if utils.ValueToInt(year,0) != yearstr{
				continue
			}
			month := int(date.Month())
			if unit == "top"{
				if month >6{
					continue
				}
			}else if unit == "bottom"{
				if month <= 6{
					continue
				}
			}else {

			}

			if joinResult.SubjectCategoryid != "整体满意程度" {
				cate := joinResult.SubjectCategoryid
				value, exist := tempSubjectMap[cate]
				if !exist {
					tempSubjectStatisticData := SubjectStaticData{}
					tempSubjectStatisticData.CateId = cate
					switch joinResult.SelectedOptions {
					case "A":
						tempSubjectStatisticData.ASum++
					case "B":
						tempSubjectStatisticData.BSum++
					case "C":
						tempSubjectStatisticData.CSum++
					case "D":
						tempSubjectStatisticData.DSum++
					case "E":
						tempSubjectStatisticData.ESum++
					}
					tempSubjectStatisticData.Count++
					tempSubjectMap[cate] = tempSubjectStatisticData
				} else {
					switch joinResult.SelectedOptions {
					case "A":
						value.ASum++
					case "B":
						value.BSum++
					case "C":
						value.CSum++
					case "D":
						value.DSum++
					case "E":
						value.ESum++
					}
					value.Count++
					tempSubjectMap[cate] = value
				}

			} else {
				key := utils.ValueToString(cid, "") + "-" + joinResult.Buildingid
				_, ok := cidBuildMap[key]
				if !ok {
					cidBuildMap[key] += joinResult.Totalhouseholds
					totalData.SumTotal += cidBuildMap[key]
				}

				totalData.Cid = cid
				totalData.CidName = joinResult.Descr
				// 处理 整体满意度
				totalData.Count++
				switch joinResult.SelectedOptions {
				case "A":
					totalData.ASum++
				case "B":
					totalData.BSum++
				case "C":
					totalData.CSum++
				case "D":
					totalData.DSum++
				case "E":
					totalData.ESum++
				}
				oneCid.Cid = cid
				oneCid.CidName = joinResult.Descr
				oneCid.Cate = joinResult.SubjectCategoryid
				// 处理 整体满意度
				oneCid.Count++
				switch joinResult.SelectedOptions {
				case "A":
					oneCid.ASum++
				case "B":
					oneCid.BSum++
				case "C":
					oneCid.CSum++
				case "D":
					oneCid.DSum++
				case "E":
					oneCid.ESum++
				}
			}
		}
		SubjectStaticDataLis := make([]SubjectStaticData, 0)
		for pos, key := range menus {
			data, ok := tempSubjectMap[key]
			if ok {
				if data.Count == 0 {
					data.APercent = 0
					data.BPercent = 0
					data.CPercent = 0
					data.DPercent = 0
					data.EPercent = 0
				} else {
					data.APercent = utils.Decimal(float64(data.ASum)/float64(data.Count), 3)
					data.BPercent = utils.Decimal(float64(data.BSum)/float64(data.Count), 3)
					data.CPercent = utils.Decimal(float64(data.CSum)/float64(data.Count), 3)
					data.DPercent = utils.Decimal(float64(data.DSum)/float64(data.Count), 3)
					data.EPercent = utils.Decimal(float64(data.ESum)/float64(data.Count), 3)
				}
				//集团记录
				value, exist := groupSubjectMap[key]
				if !exist {
					var groupSubjectStatisticData SubjectStaticData
					groupSubjectStatisticData.CateId = key
					groupSubjectStatisticData.Pos = pos
					groupSubjectStatisticData.ASum = data.ASum
					groupSubjectStatisticData.BSum = data.BSum
					groupSubjectStatisticData.CSum = data.CSum
					groupSubjectStatisticData.DSum = data.DSum
					groupSubjectStatisticData.ESum = data.ESum
					groupSubjectStatisticData.Count = data.Count
					groupSubjectMap[key] = groupSubjectStatisticData
				} else {
					value.ASum += data.ASum
					value.BSum += data.BSum
					value.CSum += data.CSum
					value.DSum += data.DSum
					value.ESum += data.ESum
					value.Count += data.Count
					groupSubjectMap[key] = value
				}
				SubjectStaticDataLis = append(SubjectStaticDataLis, data)
			} else {
				one := SubjectStaticData{}
				one.CateId = key
				SubjectStaticDataLis = append(SubjectStaticDataLis, one)
				//集团记录
				_, exist := groupSubjectMap[key]
				if !exist {
					var groupSubjectStatisticData SubjectStaticData
					groupSubjectStatisticData.CateId = key
					groupSubjectStatisticData.Pos = pos
					groupSubjectMap[key] = groupSubjectStatisticData
				}
			}
		}

		// oneCid.SubjectStaticDataLi 排序
		sli := make([]ICompare, 0)
		for _, data := range SubjectStaticDataLis {
			sli = append(sli, data)
		}
		sli = BubbleSort(sli)
		SubjectStaticDataSort := make([]SubjectStaticData, 0)
		for _, data := range sli {
			SubjectStaticDataSort = append(SubjectStaticDataSort, data.(SubjectStaticData))
		}
		oneCid.SubjectStaticDataLi = SubjectStaticDataSort

		if totalData.Count == 0 {
			totalData.APercent = 0
			totalData.BPercent = 0
			totalData.CPercent = 0
			totalData.DPercent = 0
			totalData.EPercent = 0
			totalData.ActPersons = 0
		} else {
			totalData.APercent = utils.Decimal(float64(totalData.ASum)/float64(totalData.Count), 3)
			totalData.BPercent = utils.Decimal(float64(totalData.BSum)/float64(totalData.Count), 3)
			totalData.CPercent = utils.Decimal(float64(totalData.CSum)/float64(totalData.Count), 3)
			totalData.DPercent = utils.Decimal(float64(totalData.DSum)/float64(totalData.Count), 3)
			totalData.EPercent = utils.Decimal(float64(totalData.ESum)/float64(totalData.Count), 3)
			totalData.ActPersons = totalData.Count
			if totalData.SumTotal == 0 {
				totalData.ActPer = 0
			} else {
				totalData.ActPer = utils.Decimal(float64(totalData.ActPersons)/float64(totalData.SumTotal), 3)
			}
		}

		if oneCid.Count == 0 {
			oneCid.APercent = 0
			oneCid.BPercent = 0
			oneCid.CPercent = 0
			oneCid.DPercent = 0
			oneCid.EPercent = 0
			oneCid.ActPersons = 0
		} else {
			oneCid.APercent = utils.Decimal(float64(oneCid.ASum)/float64(oneCid.Count), 3)
			oneCid.BPercent = utils.Decimal(float64(oneCid.BSum)/float64(oneCid.Count), 3)
			oneCid.CPercent = utils.Decimal(float64(oneCid.CSum)/float64(oneCid.Count), 3)
			oneCid.DPercent = utils.Decimal(float64(oneCid.DSum)/float64(oneCid.Count), 3)
			oneCid.EPercent = utils.Decimal(float64(oneCid.ESum)/float64(oneCid.Count), 3)
			oneCid.ActPersons = totalData.Count
		}

		oneGroup.ASum += totalData.ASum
		oneGroup.BSum += totalData.BSum
		oneGroup.CSum += totalData.CSum
		oneGroup.DSum += totalData.DSum
		oneGroup.ESum += totalData.ESum
		oneGroup.Count += totalData.Count
		oneGroup.ActPersons += totalData.ActPersons
		oneGroup.SumTotal += totalData.SumTotal

		oneCidGroup.ASum += oneCid.ASum
		oneCidGroup.BSum += oneCid.BSum
		oneCidGroup.CSum += oneCid.CSum
		oneCidGroup.DSum += oneCid.DSum
		oneCidGroup.ESum += oneCid.ESum
		oneCidGroup.Count += oneCid.Count
		oneCidGroup.ActPersons += oneCid.ActPersons

		TotalStatisticData = append(TotalStatisticData, totalData)
		CidStatisticDatas = append(CidStatisticDatas, oneCid)
	}
	//集团统计
	if oneGroup.Count == 0 {
		oneGroup.APercent = 0
		oneGroup.BPercent = 0
		oneGroup.CPercent = 0
		oneGroup.DPercent = 0
		oneGroup.EPercent = 0
		oneGroup.ActPersons = 0
	} else {
		oneGroup.APercent = utils.Decimal(float64(oneGroup.ASum)/float64(oneGroup.Count), 3)
		oneGroup.BPercent = utils.Decimal(float64(oneGroup.BSum)/float64(oneGroup.Count), 3)
		oneGroup.CPercent = utils.Decimal(float64(oneGroup.CSum)/float64(oneGroup.Count), 3)
		oneGroup.DPercent = utils.Decimal(float64(oneGroup.DSum)/float64(oneGroup.Count), 3)
		oneGroup.EPercent = utils.Decimal(float64(oneGroup.ESum)/float64(oneGroup.Count), 3)
		oneGroup.ActPersons = oneGroup.Count
		if oneGroup.SumTotal == 0 {
			oneGroup.ActPer = 0
		} else {
			oneGroup.ActPer = utils.Decimal(float64(oneGroup.ActPersons)/float64(oneGroup.SumTotal), 3)
		}
	}

	//集团统计
	if oneCidGroup.Count == 0 {
		oneCidGroup.APercent = 0
		oneCidGroup.BPercent = 0
		oneCidGroup.CPercent = 0
		oneCidGroup.DPercent = 0
		oneCidGroup.EPercent = 0
		oneCidGroup.ActPersons = 0
	} else {
		oneCidGroup.APercent = utils.Decimal(float64(oneCidGroup.ASum)/float64(oneCidGroup.Count), 3)
		oneCidGroup.BPercent = utils.Decimal(float64(oneCidGroup.BSum)/float64(oneCidGroup.Count), 3)
		oneCidGroup.CPercent = utils.Decimal(float64(oneCidGroup.CSum)/float64(oneCidGroup.Count), 3)
		oneCidGroup.DPercent = utils.Decimal(float64(oneCidGroup.DSum)/float64(oneCidGroup.Count), 3)
		oneCidGroup.EPercent = utils.Decimal(float64(oneCidGroup.ESum)/float64(oneCidGroup.Count), 3)
		oneCidGroup.ActPersons = oneCidGroup.Count
	}

	groupSubjectStaticData := make([]SubjectStaticData, 0)

	for _, data := range groupSubjectMap {
		if data.Count == 0 {
			data.APercent = 0
			data.BPercent = 0
			data.CPercent = 0
			data.DPercent = 0
			data.EPercent = 0
		} else {
			data.APercent = utils.Decimal(float64(data.ASum)/float64(data.Count), 3)
			data.BPercent = utils.Decimal(float64(data.BSum)/float64(data.Count), 3)
			data.CPercent = utils.Decimal(float64(data.CSum)/float64(data.Count), 3)
			data.DPercent = utils.Decimal(float64(data.DSum)/float64(data.Count), 3)
			data.EPercent = utils.Decimal(float64(data.ESum)/float64(data.Count), 3)
		}
		groupSubjectStaticData = append(groupSubjectStaticData, data)
	}

	// oneCid.SubjectStaticDataLi 排序
	gli := make([]ICompare, 0)
	for _, data := range groupSubjectStaticData {
		gli = append(gli, data)
	}
	gli = BubbleSort(gli)
	groupSubjectStaticDataSort := make([]SubjectStaticData, 0)
	for _, data := range gli {
		groupSubjectStaticDataSort = append(groupSubjectStaticDataSort, data.(SubjectStaticData))
	}
	oneCidGroup.SubjectStaticDataLi = groupSubjectStaticDataSort

	if selectType == "ratio" {
		// 处理整体满意度的表格
		for index, data := range TotalStatisticData {
			if data.Count == 0 {
				data.SatisfactionPercent = 0
				data.SatisfactionPersons = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum)/float64(data.Count), 3)
				data.SatisfactionPersons = utils.ValueToInt(utils.Decimal(float64(data.SatisfactionPercent)*float64(data.ActPersons), 0), 0)
			}
			TotalStatisticData[index] = data
		}
		//计算集团
		if oneGroup.Count == 0 {
			oneGroup.SatisfactionPercent = 0
			oneGroup.SatisfactionPersons = 0
		} else {
			oneGroup.SatisfactionPercent = utils.Decimal(float64(oneGroup.ASum+oneGroup.BSum)/float64(oneGroup.Count), 3)
			oneGroup.SatisfactionPersons = utils.ValueToInt(utils.Decimal(float64(oneGroup.SatisfactionPercent)*float64(oneGroup.ActPersons), 0), 0)
		}

		//评比表
		for index, data := range CidStatisticDatas {
			if data.Count == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum)/float64(data.Count), 3)
			}
			for index, vdata := range data.SubjectStaticDataLi {
				if vdata.Count == 0 {
					vdata.SatisfactionPercent = 0
				} else {
					vdata.SatisfactionPercent = utils.Decimal(float64(vdata.ASum+vdata.BSum)/float64(vdata.Count), 3)
				}
				data.SubjectStaticDataLi[index] = vdata
			}

			CidStatisticDatas[index] = data
		}

		//计算集团
		if oneCidGroup.Count == 0 {
			oneCidGroup.SatisfactionPercent = 0
		} else {
			oneCidGroup.SatisfactionPercent = utils.Decimal(float64(oneCidGroup.ASum+oneCidGroup.BSum)/float64(oneCidGroup.Count), 3)
			for index, data := range oneCidGroup.SubjectStaticDataLi {
				if data.Count == 0 {
					data.SatisfactionPercent = 0
				} else {
					data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum)/float64(data.Count), 3)
				}
				oneCidGroup.SubjectStaticDataLi[index] = data
			}
		}
	} else if selectType == "threeratio" {
		/********
		*三分比例法:“3分比例法”统计的是选择“非常满意”、“满意”和“一般”的人数占比,此三项加和作为分母
		**************/
		// 处理不同subject的满意度调差
		for index, data := range TotalStatisticData {
			if data.Count == 0 {
				data.SatisfactionPercent = 0
				data.SatisfactionPersons = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum+data.CSum)/float64(data.Count), 3)
				data.SatisfactionPersons = utils.ValueToInt(utils.Decimal(float64(data.SatisfactionPercent)*float64(data.ActPersons), 0), 0)
			}
			TotalStatisticData[index] = data
		}
		//计算集团
		if oneGroup.Count == 0 {
			oneGroup.SatisfactionPercent = 0
			oneGroup.SatisfactionPersons = 0
		} else {
			oneGroup.SatisfactionPercent = utils.Decimal(float64(oneGroup.ASum+oneGroup.BSum+oneGroup.CSum)/float64(oneGroup.Count), 3)
			oneGroup.SatisfactionPersons = utils.ValueToInt(utils.Decimal(float64(oneGroup.SatisfactionPercent)*float64(oneGroup.ActPersons), 0), 0)
		}

		//评比表
		for index, data := range CidStatisticDatas {
			if data.Count == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum+data.CSum)/float64(data.Count), 3)
			}
			for index, vdata := range data.SubjectStaticDataLi {
				if vdata.Count == 0 {
					vdata.SatisfactionPercent = 0
				} else {
					vdata.SatisfactionPercent = utils.Decimal(float64(vdata.ASum+vdata.BSum+vdata.CSum)/float64(vdata.Count), 3)
				}
				data.SubjectStaticDataLi[index] = vdata
			}

			CidStatisticDatas[index] = data
		}
		//计算集团
		if oneCidGroup.Count == 0 {
			oneCidGroup.SatisfactionPercent = 0
		} else {
			oneCidGroup.SatisfactionPercent = utils.Decimal(float64(oneCidGroup.ASum+oneCidGroup.BSum+oneCidGroup.CSum)/float64(oneCidGroup.Count), 3)
			for index, data := range oneCidGroup.SubjectStaticDataLi {
				if data.Count == 0 {
					data.SatisfactionPercent = 0
				} else {
					data.SatisfactionPercent = utils.Decimal(float64(data.ASum+data.BSum+data.CSum)/float64(data.Count), 3)
				}
				oneCidGroup.SubjectStaticDataLi[index] = data
			}
		}
	} else {
		// 处理不同subject的满意度调差
		for index, data := range TotalStatisticData {
			if data.Count == 0 {
				data.SatisfactionPercent = 0
				data.SatisfactionPersons = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum*5+data.BSum*4+data.CSum*3+data.DSum*2+data.ESum*1)/float64(data.Count*5), 3)
				data.SatisfactionPersons = utils.ValueToInt(utils.Decimal(float64(data.SatisfactionPercent)*float64(data.ActPersons), 0), 0)
			}
			TotalStatisticData[index] = data
		}
		if oneGroup.Count == 0 {
			oneGroup.SatisfactionPercent = 0
			oneGroup.SatisfactionPersons = 0
		} else {
			oneGroup.SatisfactionPercent = utils.Decimal(float64(oneGroup.ASum*5+oneGroup.BSum*4+oneGroup.CSum*3+oneGroup.DSum*2+oneGroup.ESum*1)/float64(oneGroup.Count*5), 3)
			oneGroup.SatisfactionPersons = utils.ValueToInt(utils.Decimal(float64(oneGroup.SatisfactionPercent)*float64(oneGroup.ActPersons), 0), 0)
		}

		//评比表
		for index, data := range CidStatisticDatas {
			if data.Count == 0 {
				data.SatisfactionPercent = 0
			} else {
				data.SatisfactionPercent = utils.Decimal(float64(data.ASum*5+data.BSum*4+data.CSum*3+data.DSum*2+data.ESum*1)/float64(data.Count*5), 3)
			}
			for index, vdata := range data.SubjectStaticDataLi {
				if vdata.Count == 0 {
					vdata.SatisfactionPercent = 0
				} else {
					vdata.SatisfactionPercent = utils.Decimal(float64(vdata.ASum*5+vdata.BSum*4+vdata.CSum*3+vdata.DSum*2+vdata.ESum*1)/float64(vdata.Count*5), 3)
				}
				data.SubjectStaticDataLi[index] = vdata
			}

			CidStatisticDatas[index] = data
		}

		if oneCidGroup.Count == 0 {
			oneCidGroup.SatisfactionPercent = 0
		} else {
			oneCidGroup.SatisfactionPercent = utils.Decimal(float64(oneCidGroup.ASum*5+oneCidGroup.BSum*4+oneCidGroup.CSum*3+oneCidGroup.DSum*2+oneCidGroup.ESum*1)/float64(oneCidGroup.Count*5), 3)
			for index, data := range oneCidGroup.SubjectStaticDataLi {
				if data.Count == 0 {
					data.SatisfactionPercent = 0
				} else {
					data.SatisfactionPercent = utils.Decimal(float64(data.ASum*5+data.BSum*4+data.CSum*3+data.DSum*2+data.ESum*1)/float64(data.Count*5), 3)
				}
				oneCidGroup.SubjectStaticDataLi[index] = data
			}
		}
	}
	//园区统计
	items := make([]ItemDataTotalGather, 0)
	//公共秩序
	one := ItemDataTotalGather{}
	one.SubjectCategoryid = "公共秩序"
	//维修服务
	one1 := ItemDataTotalGather{}
	one1.SubjectCategoryid = "维修服务"
	//清洁绿化
	one2 := ItemDataTotalGather{}
	one2.SubjectCategoryid = "清洁绿化"
	//客服人员
	one3 := ItemDataTotalGather{}
	one3.SubjectCategoryid = "客服人员"
	//社区文化
	one4 := ItemDataTotalGather{}
	one4.SubjectCategoryid = "社区文化"
	//停车场
	one5 := ItemDataTotalGather{}
	one5.SubjectCategoryid = "停车场"
	for _, cids := range CidStatisticDatas {
		for _, vv := range cids.SubjectStaticDataLi {
			if vv.CateId == "公共秩序" {
				info := ItemInfo{}
				info.Cid = cids.Cid
				info.CidName = cids.CidName
				info.SatisfactionPercent = vv.SatisfactionPercent
				one.ItemInfo = append(one.ItemInfo, info)
			}
			if vv.CateId == "维修服务" {
				info := ItemInfo{}
				info.Cid = cids.Cid
				info.CidName = cids.CidName
				info.SatisfactionPercent = vv.SatisfactionPercent
				one1.ItemInfo = append(one1.ItemInfo, info)
			}
			if vv.CateId == "清洁绿化" {
				info := ItemInfo{}
				info.Cid = cids.Cid
				info.CidName = cids.CidName
				info.SatisfactionPercent = vv.SatisfactionPercent
				one2.ItemInfo = append(one2.ItemInfo, info)
			}
			if vv.CateId == "客服人员" {
				info := ItemInfo{}
				info.Cid = cids.Cid
				info.CidName = cids.CidName
				info.SatisfactionPercent = vv.SatisfactionPercent
				one3.ItemInfo = append(one3.ItemInfo, info)
			}
			if vv.CateId == "社区文化" {
				info := ItemInfo{}
				info.Cid = cids.Cid
				info.CidName = cids.CidName
				info.SatisfactionPercent = vv.SatisfactionPercent
				one4.ItemInfo = append(one4.ItemInfo, info)
			}
			if vv.CateId == "停车场" {
				info := ItemInfo{}
				info.Cid = cids.Cid
				info.CidName = cids.CidName
				info.SatisfactionPercent = vv.SatisfactionPercent
				one5.ItemInfo = append(one5.ItemInfo, info)
			}
		}
	}
	items = append(items, one)
	items = append(items, one1)
	items = append(items, one2)
	items = append(items, one3)
	items = append(items, one4)
	items = append(items, one5)
	// TotalStatisticData 排序
	li := make([]ICompare, 0)
	for _, data := range TotalStatisticData {
		li = append(li, data)
	}
	li = BubbleSort(li)
	TotalDataSort := make([]GatherTotalStatisticData, 0)
	for _, data := range li {
		TotalDataSort = append(TotalDataSort, data.(GatherTotalStatisticData))
	}

	// CidStatisticData 排序
	cidli := make([]ICompare, 0)
	for _, data := range CidStatisticDatas {
		cidli = append(cidli, data)
	}
	cidli = BubbleSort(cidli)
	CidStatisticDataSort := make([]CidStatisticData, 0)
	for _, data := range cidli {
		CidStatisticDataSort = append(CidStatisticDataSort, data.(CidStatisticData))
	}

	TotalDataSort = append(TotalDataSort, oneGroup)

	// oneCidGroup.SubjectStaticDataLi 排序
	groupli := make([]ICompare, 0)
	for _, data := range oneCidGroup.SubjectStaticDataLi {
		groupli = append(groupli, data)
	}
	groupli = BubbleSort(groupli)
	oneCidGroupDataSort := make([]SubjectStaticData, 0)
	for _, data := range groupli {
		oneCidGroupDataSort = append(oneCidGroupDataSort, data.(SubjectStaticData))
	}
	oneCidGroup.SubjectStaticDataLi = oneCidGroupDataSort

	CidStatisticDataSort = append(CidStatisticDataSort, oneCidGroup)

	result.GatherProjectTotalPer = TotalDataSort
	result.GatherTotal = CidStatisticDataSort
	result.ItemDataTotalGather = items

	return result, nil
}

/******
*需求：超级管理员可以通过用户名，直接通过页面看到这份问卷的答题情况
*****************/
func (t *CSurveysheetResult) SurveysheetSearchByUser() ([]CSurveysheetResultlst, error) {
	e := db.MasterEngine()
	data := make([]CSurveysheetResultlst, 0)
	err := e.Table("c_surveysheet_resultlst").Join("INNER", "c_surveysheet_result", "c_surveysheet_resultlst.cid = c_surveysheet_result.cid and c_surveysheet_resultlst.surveynr = c_surveysheet_result.surveynr and c_surveysheet_resultlst.surveysheetid = c_surveysheet_result.surveysheetid").Where("c_surveysheet_result.cid =? and c_surveysheet_result.lastmodifyby = ? and c_surveysheet_result.surveysheetid = ?", t.Cid, t.Lastmodifyby, t.Surveysheetid).Find(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}
