package models

import (
	"SSW_WebPlatform/db"
	"SSW_WebPlatform/utils"
	"errors"

	"xorm.io/core"
)

type Stdeftab struct {
	Stdeftyp  string `xorm:"pk comment('种类') VARCHAR(15)" json:"stdeftab-stdeftyp"`
	Statid    string `xorm:"pk comment('标识') VARCHAR(30)" json:"stdeftab-statid"`
	Lang      string `xorm:"pk comment('语言') VARCHAR(3)" json:"stdeftab-lang"`
	Bez       string `xorm:"comment('名称') VARCHAR(30)" json:"stdeftab-bez"`
	Lastmodif string `xorm:"comment('最近一次更改时间') VARCHAR(14)" json:"stdeftab-lastmodif"`
	Lastuser  string `xorm:"comment('最近一次更改人') VARCHAR(20)" json:"stdeftab-lastuser"`
	Credatuz  string `xorm:"comment('创建时间') VARCHAR(14)" json:"stdeftab-credatuz"`
}

func (t *Stdeftab) TableName() string {
	return "stdeftab"
}

// 清除string字段的右侧空格
func (t *Stdeftab) Clipped() {
	utils.TrimStruct(t, *t)
}

//增
func (t *Stdeftab) Add() error {
	e := db.MasterEngine()
	count := new(Stdeftab)
	affw, err := e.Table("stdeftab").ID(core.PK{ t.Stdeftyp, t.Statid, t.Lang}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table("stdeftab").Insert(t)

	if err != nil {
		return err
	}
	return nil
}

//删
func (t *Stdeftab) Del() bool {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Stdeftyp, t.Statid, t.Lang}).Delete(&Stdeftab{})
	if err != nil {
		return false
	}
	return true
}

//改
func (t *Stdeftab) Update() bool {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Stdeftyp, t.Statid, t.Lang}).Update(t)
	if err != nil {
		return false
	}
	return true
}

//查
func (t *Stdeftab) SelectOne() (Stdeftab, error) {
	e := db.MasterEngine()
	var data Stdeftab
	_, err := e.ID(core.PK{t.Stdeftyp, t.Statid, t.Lang}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

//分页
func (t *Stdeftab) GetPage(pageSize int, pageIndex int) ([]Stdeftab, int, error) {
	data := make([]Stdeftab, 0)
	e := db.MasterEngine()
	query := e.Table("stdeftab")
	table := e.Table("stdeftab")
	Offset := (pageIndex - 1) * pageSize
	err := query.Limit(pageSize, Offset).Desc("credatuz").Find(&data)
	pcount := new(Stdeftab)
	count, err := table.Count(pcount)
	if err != nil {
		return data, 0, err
	}
	return data, int(count), nil
}

//查询当前Stdeftabs集合
func (t *Stdeftab) SelectArr() ([]Stdeftab, error) {
	e := db.MasterEngine()
	var data []Stdeftab
	err := e.Where("statid =?",  t.Statid).Find(&data)
	if err != nil {
		return data, err
	}
	for k, _ := range data {
		data[k].Clipped()
	}
	return data, nil
}

//查询当前Stdeftabs集合
func (t *Stdeftab) SelectAll(arr []string) (map[string][]Stdeftab, error) {
	e := db.MasterEngine()
	dataMap := map[string][]Stdeftab{}
	var data []Stdeftab
	err := e.In("statid", arr).Find(&data)
	if err != nil {
		return dataMap, err
	}
	for _, v := range data {
		v.Clipped()
		key := v.Statid
		dataMap[key] = append(dataMap[key], v)
	}
	return dataMap, nil
}
