// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.
package wxSmall

import (
	"SSW_WebPlatform/db"
	"SSW_WebPlatform/utils"
	"SSW_WebPlatform/web/middleware/glog"
	"SSW_WebPlatform/web/models"
	"strings"
	"time"
	"xorm.io/core"
)

type WxCacheSurveysheetResult struct {
	Cid                 int                           `json:"cid" xorm:"not null pk comment('公司ID') INT(11)"`
	Cnr                 int                           `json:"cnr" xorm:"not null pk comment('小区编号') INT(11)"`
	Surveysheetid       string                        `json:"surveysheetid" xorm:"not null pk VARCHAR(40)"`
	Surveynr            int                           `json:"surveynr" xorm:"not null pk comment('调研问卷流水号') INT(11)"`
	Uid                 int                           `json:"uid" xorm:"not  NULL pk INT(10)"`
	Room                string                        `json:"room" xorm:"default 'NULL' VARCHAR(32)"`
	Unit                string                        `json:"unit" xorm:"default 'NULL' VARCHAR(32)"`
	Buildingid          string                        `json:"buildingid" xorm:"default 'NULL' VARCHAR(32)"`
	Surveyperson        string                        `json:"surveyperson" xorm:"default 'NULL' comment('指定调研人') VARCHAR(40)"`
	Surveysamplecount   int                           `json:"surveysamplecount" xorm:"not null comment('调研取样数') INT(11)"`
	Surveyfinishedcount int                           `json:"surveyfinishedcount" xorm:"not null comment('调研问卷实际已完成数') INT(11)"`
	Planstartdate       string                        `json:"planstartdate" xorm:"default '''' VARCHAR(20)"`
	Planendate          string                        `json:"planendate" xorm:"default '''' VARCHAR(20)"`
	Actstartdate        string                        `json:"actstartdate" xorm:"default '''' VARCHAR(20)"`
	Actenddate          string                        `json:"actenddate" xorm:"default '''' VARCHAR(20)"`
	Status              int                           `json:"status" xorm:"not null comment('状态') INT(11)"`
	Remark              string                        `json:"remark" xorm:"default 'NULL' comment('备注') VARCHAR(255)"`
	Signature           string                        `json:"signature" xorm:"default 'NULL' comment('电子签名') VARCHAR(255)"`
	Createtime          string                        `json:"createtime" xorm:"default '''' comment('创建时间') VARCHAR(40)"`
	Lastmodifytime      string                        `json:"lastmodifytime" xorm:"default '''' comment('最近更新时间') VARCHAR(40)"`
	Lastmodifyby        string                        `json:"lastmodifyby" xorm:"default 'NULL' comment('修改人员') VARCHAR(40)"`
	Sex                 string                        `json:"sex" xorm:"default '''' comment('被访问者性别 male 和female') VARCHAR(10)"`
	Persontype          int                           `json:"persontype" xorm:"default NULL comment('被访问者类型 业主1 租户2') TINYINT(1)"`
	Age                 int                           `json:"age" xorm:"default NULL comment('被访问者年龄 1-22岁以下 2-22到59岁 3-60岁以上') TINYINT(1)"`
	Livetime            int                           `json:"livetime" xorm:"default NULL comment('被访问者居住时间 1-1年以内 2-1到3年 2-3到5年 3-5年以上') TINYINT(1)"`
	Personname          string                        `json:"personname" xorm:"default '''' comment('被访问者姓名') VARCHAR(40)"`
	Valst               []WxCacheSurveysheetResultlst `json:"valst" xorm:"-"`
}

/******数据表名******/
func (t *WxCacheSurveysheetResult) TableName() string {
	return "wx_cache_surveysheet_result"
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据添加
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResult) Add(data models.CSurveysheetResult) error {
	engine := db.MasterEngine()
	session := engine.NewSession()
	defer session.Close()
	// add Begin() before any action
	err := session.Begin()
	if err != nil {
		return err
	}
	//查询信息
	subject := new(models.CSurveysheet)
	subject.Cid = data.Cid
	subject.Surveysheetid = data.Surveysheetid
	subject.Cnr = data.Cnr
	_, err = engine.Table("c_surveysheet").ID(core.PK{subject.Cid, subject.Cnr, subject.Surveysheetid}).Get(subject)
	if err != nil {
		session.Rollback()
		return err
	}
	//判断是否已经添加缓存
	cache := new(WxCacheSurveysheetResult)
	ok,err := session.Table("wx_cache_surveysheet_result").Where("cid =? and uid = ? and surveysheetid = ?", t.Cid, t.Uid, t.Surveysheetid).Get(cache)
	if err != nil {
		session.Rollback()
		return err
	}

	surveynr := 0
	if ok{
		//更新
		surveynr = data.Surveynr
	}else{
		snr := new(models.Snrtab)
		snr.Cid = data.Cid
		id, err := snr.GetNextSnr("Cachenr")
		if err != nil {
			session.Rollback()
			return err
		}
		surveynr = utils.ValueToInt(id, 0)
		//添加答题
		Survey := new(WxCacheSurveysheetResult)
		Survey.Surveyperson = subject.Surveyperson
		Survey.Surveysamplecount = subject.Surveysamplecount
		Survey.Surveyfinishedcount = subject.Surveyfinishedcount
		Survey.Cid = data.Cid
		Survey.Cnr = data.Cnr
		Survey.Uid = t.Uid
		Survey.Surveysheetid = data.Surveysheetid
		Survey.Buildingid = data.Buildingid
		Survey.Unit = data.Unit
		Survey.Room = data.Room
		Survey.Surveynr = surveynr
		Survey.Remark = data.Remark
		Survey.Signature = data.Signature
		Survey.Planstartdate = data.Planstartdate
		Survey.Planendate = data.Planendate
		Survey.Actenddate = utils.TimeFormat(time.Now(), "yyyy-MM-dd HH:mm:ss")
		//查询是否存在，

		_, err = session.Table("wx_cache_surveysheet_result").Insert(Survey)
		if err != nil {
			glog.InfoExt("问题调研", "添加题目 err1 is :", err)
			session.Rollback()
			return err
		}
	}


	//记录答案
	for _, v := range data.Valst {
		//查询答案分值
		datalist := make([]models.CSurveysheetOptionlst, 0)
		SelectedOptions := strings.Split(v.SelectedOptions, ",")
		glog.InfoExt("问题调研", "添加问题 SelectedOptionsis :", SelectedOptions)
		err = engine.Table("c_surveysheet_optionlst").Where("cid = ? and subjectid = ?", data.Cid, v.Subjectid).In("optionid", SelectedOptions).Find(&datalist)
		glog.InfoExt("问题调研", "添加问题 datalist :", datalist)
		for _, vv := range datalist {

			//判断是否已经添加缓存
			cachelst := new(WxCacheSurveysheetResultlst)
			ok,err := session.Table("wx_cache_surveysheet_resultlst").Where("cid =? and uid = ? and surveysheetid = ? and surveynr = ? and subjectid = ?", t.Cid, t.Uid, t.Surveysheetid,surveynr,vv.Subjectid).Get(cachelst)
			if err != nil {
				session.Rollback()
				return err
			}
			if ok{
				//更新
				subject := new(WxCacheSurveysheetResultlst)
				subject.Cid = data.Cid
				subject.Cnr = data.Cnr
				subject.Uid = t.Uid
				subject.Subjectid = vv.Subjectid
				subject.Surveynr = surveynr
				subject.Surveysheetid = data.Surveysheetid
				subject.Status = v.Status
				subject.SelectedOptions = vv.Optionid
				subject.Assessmentvalue = vv.Optionvalue
				subject.Remark = v.Remark
				subject.Optiontype = vv.Optiontype
				subject.Optioninput = v.Optioninput
				subject.Lastmodifyby = data.Lastmodifyby
				subject.Createtime = data.Createtime
				subject.Lastmodifytime = data.Lastmodifytime
				_, err = session.Table("wx_cache_surveysheet_resultlst").Where("cid =? and uid = ? and surveysheetid = ? and surveynr = ? and subjectid = ?", t.Cid, t.Uid, t.Surveysheetid,surveynr,vv.Subjectid).Update(subject)
				if err != nil {
					glog.InfoExt("问题调研", "添加问题 err2 is :", err)
					session.Rollback()
					return err
				}
			}else{
				subject := new(WxCacheSurveysheetResultlst)
				subject.Cid = data.Cid
				subject.Cnr = data.Cnr
				subject.Uid = t.Uid
				subject.Subjectid = vv.Subjectid
				subject.Surveynr = surveynr
				subject.Surveysheetid = data.Surveysheetid
				subject.Status = v.Status
				subject.SelectedOptions = vv.Optionid
				subject.Assessmentvalue = vv.Optionvalue
				subject.Remark = v.Remark
				subject.Optiontype = vv.Optiontype
				subject.Optioninput = v.Optioninput
				subject.Lastmodifyby = data.Lastmodifyby
				subject.Createtime = data.Createtime
				subject.Lastmodifytime = data.Lastmodifytime
				_, err = session.Table("wx_cache_surveysheet_resultlst").Insert(subject)
				if err != nil {
					glog.InfoExt("问题调研", "添加问题 err2 is :", err)
					session.Rollback()
					return err
				}
			}

		}

	}

	err = session.Commit()
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResult) Del() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Cid, t.Cnr, t.Surveysheetid, t.Surveynr}).Delete(&WxCacheSurveysheetResult{})
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据修改
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResult) Update() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Cid, t.Cnr, t.Surveysheetid, t.Surveynr}).Update(t)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResult) SelectOne() (WxCacheSurveysheetResult, error) {
	e := db.MasterEngine()
	var data WxCacheSurveysheetResult
	_, err := e.ID(core.PK{t.Cid, t.Cnr, t.Surveysheetid, t.Surveynr}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}
