// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.
package wxSmall

import (
	"errors"
	"SSW_WebPlatform/db"
	"xorm.io/core"
)

type WxCacheSurveysheetResultlst struct {
	Cid             int    `json:"cid" xorm:"not null pk comment('公司ID') INT(11)"`
	Cnr             int    `json:"cnr" xorm:"not null pk comment('小区编号') INT(11)"`
	Surveynr        int    `json:"surveynr" xorm:"not null pk comment('流水号') INT(11)"`
	Surveysheetid   string `json:"surveysheetid" xorm:"not null pk VARCHAR(40)"`
	Subjectid       string `json:"subjectid" xorm:"not null pk comment('问题ID') VARCHAR(20)"`
	Uid             int    `json:"uid" xorm:"default NULL INT(10)"`
	Optioninput     string `json:"optioninput" xorm:"default 'NULL' VARCHAR(255)"`
	Optiontype      int    `json:"optiontype" xorm:"default NULL INT(8)"`
	Status          int    `json:"status" xorm:"not null comment('状态') INT(11)"`
	SelectedOptions string `json:"selected_options" xorm:"not null pk comment('用户选择项') VARCHAR(40)"`
	Assessmentvalue int    `json:"assessmentvalue" xorm:"default NULL comment('评估值') INT(11)"`
	Remark          string `json:"remark" xorm:"default 'NULL' comment('备注') VARCHAR(255)"`
	Createtime      string `json:"createtime" xorm:"default '''' comment('创建时间') VARCHAR(40)"`
	Lastmodifytime  string `json:"lastmodifytime" xorm:"default '''' comment('最近更新时间') VARCHAR(40)"`
	Lastmodifyby    string `json:"lastmodifyby" xorm:"default 'NULL' comment('修改人员') VARCHAR(40)"`
}

/******数据表名******/
func (t *WxCacheSurveysheetResultlst) TableName() string {
	return "wx_cache_surveysheet_resultlst"
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据添加
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResultlst) Add() error {
	e := db.MasterEngine()
	count := new(WxCacheSurveysheetResultlst)
	affw, err := e.Table(t.TableName()).ID(core.PK{t.Cid, t.Cnr, t.Surveynr, t.Surveysheetid, t.Subjectid, t.SelectedOptions}).Count(count)
	if err != nil {
		return err
	}
	if affw > 0 {
		return errors.New("数据已经存在！")
	}
	_, err = e.Table(t.TableName()).Insert(t)

	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResultlst) Del() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Cid, t.Cnr, t.Surveynr, t.Surveysheetid, t.Subjectid, t.SelectedOptions}).Delete(&WxCacheSurveysheetResultlst{})
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据修改
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResultlst) Update() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Cid, t.Cnr, t.Surveynr, t.Surveysheetid, t.Subjectid, t.SelectedOptions}).Update(t)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxCacheSurveysheetResultlst) SelectOne() (WxCacheSurveysheetResultlst, error) {
	e := db.MasterEngine()
	var data WxCacheSurveysheetResultlst
	_, err := e.ID(core.PK{t.Cid, t.Cnr, t.Surveynr, t.Surveysheetid, t.Subjectid, t.SelectedOptions}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}
