// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.
package wxSmall

import (
	"SSW_WebPlatform/db"
	"xorm.io/core"
)

type WxUser struct {
	Uid            int    `json:"uid" xorm:"not null pk autoincr INT(11)"`
	Openid         string `json:"openid" xorm:"not null VARCHAR(128)"`
	Cid            int    `json:"cid" xorm:"default NULL INT(11)"`
	Name           string `json:"name" xorm:"default 'NULL' VARCHAR(32)"`
	Phone          string `json:"phone" xorm:"default 'NULL' VARCHAR(16)"`
	Nickname       string `json:"nickname" xorm:"default 'NULL' VARCHAR(32)"`
	Gender         int    `json:"gender" xorm:"default NULL comment('性别') INT(1)"`
	Language       string `json:"language" xorm:"default 'NULL' VARCHAR(16)"`
	City           string `json:"city" xorm:"default 'NULL' VARCHAR(16)"`
	Province       string `json:"province" xorm:"default 'NULL' VARCHAR(16)"`
	Country        string `json:"country" xorm:"default 'NULL' VARCHAR(16)"`
	Avatarurl      string `json:"avatarurl" xorm:"default 'NULL' VARCHAR(255)"`
	Unionid        string `json:"unionid" xorm:"default 'NULL' VARCHAR(128)"`
	Sessionkey     string `json:"sessionkey" xorm:"default 'NULL' VARCHAR(128)"`
	Building       string `json:"building" xorm:"default 'NULL' VARCHAR(10)"`
	Uint           string `json:"uint" xorm:"default 'NULL' VARCHAR(10)"`
	Room           string `json:"room" xorm:"default 'NULL' VARCHAR(10)"`
	Remark         string `json:"remark" xorm:"default 'NULL' VARCHAR(128)"`
	Createtime     string `json:"createtime" xorm:"default '''' comment('创建时间') VARCHAR(40)"`
	Lastmodifytime string `json:"lastmodifytime" xorm:"default '''' comment('最近更新时间') VARCHAR(40)"`
	Lastmodifyby   string `json:"lastmodifyby" xorm:"default 'NULL' comment('修改人员') VARCHAR(40)"`
}

type UserInfo struct {
	NickName  string `json:"nickName"`
	Country   string `json:"country"`
	Province  string `json:"province"`
	City      string `json:"city"`
	Language  string `json:"language"`
	Gender    int    `json:"gender"`
	AvatarUrl string `json:"avatarUrl"`
}

type UserData struct {
	UserInfo      UserInfo
	RawData       string `json:"rawData"`
	Signatrue     string `json:"signature"`
	EncryptedData string `json:"encryptedData"`
	Iv            string `json:"iv"`
	CloudId       string `json:"cloudID"`
	Code          string `json:"code"`
}

/******数据表名******/
func (t *WxUser) TableName() string {
	return "wx_user"
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据添加
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUser) Add() error {
	e := db.MasterEngine()
	_, err := e.Table(t.TableName()).Insert(t)

	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUser) Del() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Uid}).Delete(&WxUser{})
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据修改
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUser) Update() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Uid}).Update(t)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUser) SelectOne() (WxUser, error) {
	e := db.MasterEngine()
	var data WxUser
	_, err := e.ID(core.PK{t.Uid}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUser) SelectOneByOpenid() (*WxUser, error) {
	e := db.MasterEngine()
	data := new(WxUser)
	ok, err := e.Where("openid = ?", t.Openid).Get(data)
	if err != nil {
		return nil, err
	}
	if !ok {
		return nil, err
	}
	return data, nil
}
