// Copyright (c) Shenyang Leading Edge Intelligent Technology Co., Ltd. All rights reserved.
package wxSmall

import (
	"SSW_WebPlatform/db"
	"xorm.io/core"
)

type WxUserCache struct {
	Id             int    `json:"id" xorm:"not null pk autoincr INT(10)"`
	Uid            int    `json:"uid" xorm:"default NULL INT(10)"`
	Cid            int    `json:"cid" xorm:"default NULL INT(6)"`
	Surveysheetid  string `json:"surveysheetid" xorm:"default NULL VARCHAR(40)"`
	Status         int    `json:"status" xorm:"default NULL INT(1)"`
	Createtime     string `json:"createtime" xorm:"default '''' comment('创建时间') VARCHAR(40)"`
	Lastmodifytime string `json:"lastmodifytime" xorm:"default '''' comment('最近更新时间') VARCHAR(40)"`
	Lastmodifyby   string `json:"lastmodifyby" xorm:"default 'NULL' comment('修改人员') VARCHAR(40)"`
}

/******数据表名******/
func (t *WxUserCache) TableName() string {
	return "wx_user_cache"
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据添加
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) Add() error {
	e := db.MasterEngine()
	_, err := e.Table(t.TableName()).Insert(t)

	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) Del() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Id}).Delete(&WxUserCache{})
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据修改
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) Update() error {
	e := db.MasterEngine()
	_, err := e.ID(core.PK{t.Id}).Update(t)
	if err != nil {
		return err
	}
	return nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) SelectOne() (WxUserCache, error) {
	e := db.MasterEngine()
	var data WxUserCache
	_, err := e.ID(core.PK{t.Id}).Get(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据查找
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) LoadInfo() (WxUserCache, error) {
	e := db.MasterEngine()
	var data WxUserCache
	ok, err := e.Where("cid =? and uid = ? and surveysheetid = ?", t.Cid, t.Uid, t.Surveysheetid).Get(&data)
	if err != nil {
		return data, err
	}
	if ok {
		return data, nil
	}
	_, err = e.Table(t.TableName()).Insert(t)
	if err != nil {
		return data, err
	}
	return *t, nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 查询缓存数据
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) SelectCacheInfo() ([]WxCacheSurveysheetResultlst, error) {
	e := db.MasterEngine()
	data := make([]WxCacheSurveysheetResultlst, 0)
	err := e.Table("wx_cache_surveysheet_resultlst").Join("INNER", "wx_user_cache", "wx_cache_surveysheet_resultlst.cid = wx_user_cache.cid and wx_cache_surveysheet_resultlst.uid = wx_user_cache.uid and wx_cache_surveysheet_resultlst.surveysheetid = wx_user_cache.surveysheetid").Where("wx_user_cache.cid =? and wx_user_cache.uid = ? and wx_user_cache.surveysheetid = ? and wx_user_cache.status = ?", t.Cid, t.Uid, t.Surveysheetid,0).Find(&data)
	if err != nil {
		return data, err
	}
	return data, nil
}

/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 数据删除缓存
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) DelCache() error {
	e := db.MasterEngine()
	//更新状态为已经完成
	_, err := e.Table("wx_user_cache").Where("cid =? and uid = ? and surveysheetid = ?", t.Cid, t.Uid, t.Surveysheetid).Update(map[string]interface{}{"status":1})
	if err != nil {
		return err
	}
	_, err = e.Table("wx_cache_surveysheet_result").Where("cid =? and uid = ? and surveysheetid = ?", t.Cid, t.Uid, t.Surveysheetid).Delete(&WxCacheSurveysheetResult{})
	if err != nil {
		return err
	}
	_, err = e.Table("wx_cache_surveysheet_resultlst").Where("cid =? and uid = ? and surveysheetid = ?", t.Cid, t.Uid, t.Surveysheetid).Delete(&WxCacheSurveysheetResultlst{})
	if err != nil {
		return err
	}
	return nil
}



/******************************************************************************
 *
 * @Function Name      :
 *-----------------------------------------------------------------------------
 *
 * @Description        : 查询缓存数据
 *
 * @Function Parameters:
 *
 * @Return Value       :
 *
 * @Author             : Lou Wenzhi
 *
 * @Date               : 2021/3/6 8:47
 *
 ******************************************************************************/
func (t *WxUserCache) SelectResultInfo() (bool, error) {
	e := db.MasterEngine()
	var data WxUserCache
	ok,err := e.Table("wx_user_cache").Where("cid =? and uid = ? and surveysheetid = ? and status = ?", t.Cid, t.Uid, t.Surveysheetid,1).Get(&data)
	if err != nil {
		return false, err
	}
	return ok, nil
}