package routes

import (
	"SSW_WebPlatform/web/controllers/wxSmall"
	"github.com/kataras/iris/v12"
	"SSW_WebPlatform/web/controllers"
	"SSW_WebPlatform/web/middleware"
	"SSW_WebPlatform/web/middleware/cors"
)

// 所有的路由
func Hub(app *iris.Application) {

	/* 定义路由 */
	main := app.Party("/", cors.Mycors()).AllowMethods(iris.MethodOptions)
	//中间件，验证token和权限路径
	main.Use(middleware.ServeHTTP)

	home := main.Party("/")
	home.Get("/sysMenu", controllers.DynamicMenu) // 获取动态菜单

	// 用户API模块
	user := main.Party("/user")
	user.Post("/login", controllers.Login)            //登录
	user.Get("/courts", controllers.CourtsTable)      //项目列表
	user.Get("/getuserinfo", controllers.GetUserInfo) // 返回用户信息
	//个人中心
	user.Get("/profile", controllers.GetSysUserProfile)  //获取当前登录用户
	user.Get("/getimg", controllers.GetImg)              //重定向获取图片
	user.Post("/avatar", controllers.InsetSysUserAvatar) //上传图片
	user.Put("/pwd", controllers.SysUserUpdatePwd)       //更新密码

	//// 权限API模块
	admin := main.Party("/admin")
	{
		// 用户管理
		users := admin.Party("/users")
		users.Get("/userlist", controllers.GetSysUserList)                  // 用户列表
		users.Get("/user/{userId:string}", controllers.GetSysUser)          //获取用户
		users.Get("/urole", controllers.GetSysUserInit)                     //获取用户角色
		users.Get("/alluser", controllers.GetAllUsers)                      //获取所有用户名
		users.Post("/adduser", controllers.InsertSysUser)                   //添加用户
		users.Put("/upuser", controllers.UpdateSysUser)                     //更新用户
		users.Delete("/deluser/{userId:string}", controllers.DeleteSysUser) //删除用户

		//角色管理
		role := admin.Party("/role")
		role.Get("/rolelist", controllers.GetRoleList)                  // 角色报表
		role.Get("/rolebyid", controllers.GetRole)                      // 查询角色
		role.Post("/addrole", controllers.InsertRole)                   // 创建角色
		role.Put("/uprole", controllers.UpdateRole)                     // 更新角色
		role.Delete("/delrole/{roleId:string}", controllers.DeleteRole) // 删除角色

		//菜单管理
		menu := admin.Party("/menu")
		menu.Get("/menulist", controllers.GetMenuList)                                  //菜单列表
		menu.Get("/menubyid", controllers.GetMenu)                                      //by id查询
		menu.Post("/addmenu", controllers.InsertMenu)                                   //添加菜单
		menu.Put("/upmenu", controllers.UpdateMenu)                                     //更新菜单
		menu.Delete("/delmenu/{id:int}", controllers.DeleteMenu)                        //删除菜单
		menu.Get("/menuids", controllers.GetMenuIDS)                                    //通过ids查询菜单
		menu.Get("/menuTreeselect", controllers.GetMenuTreeelect)                       //获取菜单列表
		menu.Get("/roleMenuTreeselect/{roleId:int}", controllers.GetMenuTreeRoleselect) //获取菜单

		// 日志管理
		logs := admin.Party("/logs")
		logs.Get("/loglist", controllers.Loglist)

		// 翻译列表
		tabnames := admin.Party("/tabnames")
		tabnames.Post("/addinfo", controllers.TabInfo)
		tabnames.Get("/listpage", controllers.TabDataListPage)
		tabnames.Get("/list", controllers.TabDataList)
		tabnames.Get("/download", controllers.Download)
		tabnames.Delete("/del", controllers.DeleteTab)

		// 标准定义
		stdeftab := admin.Party("/stdeftab")
		stdeftab.Get("/listpage", controllers.GetStdeftabList)
		stdeftab.Get("/selectinfo", controllers.GetStdeftab)
		stdeftab.Post("/addinfo", controllers.InsertStdeftab)
		stdeftab.Delete("/del", controllers.DeleteStdeftab)
		stdeftab.Put("/upinfo", controllers.UpdateStdeftab)
		stdeftab.Get("/selectarr", controllers.SelectStdeftabArr)
		stdeftab.Get("/selectall", controllers.SelectStdeftabAll)

		// 订单流水号
		snrtab := admin.Party("/snrtab")
		snrtab.Get("/listpage", controllers.GetSnrtabList)
		snrtab.Get("/selectinfo", controllers.GetSnrtab)
		snrtab.Post("/addinfo", controllers.InsertSnrtab)
		snrtab.Delete("/del", controllers.DeleteSnrtab)
		snrtab.Put("/upinfo", controllers.UpdateSnrtab)

		// 导入表
		scripttab := admin.Party("/scripttab")
		scripttab.Get("/listpage", controllers.GetScripttabList)
		scripttab.Get("/selectinfo", controllers.GetScripttab)
		scripttab.Post("/addinfo", controllers.InsertScripttab)
		scripttab.Delete("/del", controllers.DeleteScripttab)
		scripttab.Put("/upinfo", controllers.UpdateScripttab)
		scripttab.Post("/upexcel", controllers.UpExcelScripttab)
		scripttab.Get("/selectarr", controllers.SelectArrScripttab)

		// 项目
		courttab := admin.Party("/courttab")
		courttab.Get("/listpage", controllers.GetCourttabList)
		courttab.Get("/selectinfo", controllers.GetCourttab)
		courttab.Put("/upinfo", controllers.UpdateCourttab)
		courttab.Post("/addinfo", controllers.InsertCourttab)
		courttab.Delete("/del", controllers.DeleteCourttab)

		//公司信息
		company := admin.Party("/company")
		company.Get("/listpage", controllers.GetCompanyList)
		company.Get("/selectinfo", controllers.GetCompany)
		company.Put("/upinfo", controllers.UpdateCompany)
		company.Post("/addinfo", controllers.InsertCompany)
		company.Delete("/del", controllers.DeleteCompany)

		//问题分类
		subject_category := admin.Party("/subject_category")
		subject_category.Get("/listpage", controllers.GetSubjectCategoryList)
		subject_category.Get("/selectinfo", controllers.GetSubjectCategory)
		subject_category.Get("/selectarr", controllers.AllSubjectCategory)
		subject_category.Put("/upinfo", controllers.UpdateSubjectCategory)
		subject_category.Post("/addinfo", controllers.InsertSubjectCategory)
		subject_category.Delete("/del", controllers.DeleteSubjectCategory)

		//问题
		subject := admin.Party("/subject")
		subject.Get("/listpage", controllers.GetSubjectList)
		subject.Post("/media", controllers.UploadSubject)
		subject.Get("/selectinfo", controllers.GetSubject)
		subject.Put("/upinfo", controllers.UpdateSubject)
		subject.Post("/addinfo", controllers.InsertSubject)
		subject.Delete("/del", controllers.DeleteSubject)
		subject.Get("/selectarr", controllers.SubjectArr)

		//调查问卷
		c_surveysheet := admin.Party("/c_surveysheet")
		c_surveysheet.Get("/listpage", controllers.GetCSurveysheetList)
		c_surveysheet.Get("/selectinfo", controllers.GetCSurveysheet)
		c_surveysheet.Put("/upinfo", controllers.UpdateCSurveysheet)
		c_surveysheet.Post("/addinfo", controllers.InsertCSurveysheet)
		c_surveysheet.Delete("/del", controllers.DeleteCSurveysheet)
		c_surveysheet.Get("/selectresultbyuser", controllers.GetCSurveysheetByUser)

		//用户答题
		c_surveysheet_result := admin.Party("/c_surveysheet_result")
		c_surveysheet_result.Post("/addinfo", controllers.InsertCSurveysheetResult)
		c_surveysheet_result.Get("/total", controllers.TotalCSurveysheetResult)
		c_surveysheet_result.Get("/totalmultiple", controllers.TotalMultipleCSurveysheetResult)
		c_surveysheet_result.Get("/selectarr", controllers.SelectArrCSurveysheet)
		c_surveysheet_result.Get("/statistic", controllers.StatisticData)
		c_surveysheet_result.Get("/allstatistic", controllers.AllStatisticData)
	}

	// 权限API模块
	api := main.Party("/api")
	api.Get("/getbuildings", wxSmall.GetBuildings) //查询楼栋
	api.Post("/wxlogin", wxSmall.Wxlogin)          //登录
	{
		wx_surveysheet := api.Party("/wx_surveysheet")
		wx_surveysheet.Get("/checkresult", wxSmall.WxCheckResult)                 //是否做过调查
		wx_surveysheet.Get("/checkbarcode", wxSmall.WxCheckBarcode)               //判断二维码是否过期
		wx_surveysheet.Get("/selectinfo", wxSmall.GetWxCSurveysheet)              //查询试卷
		wx_surveysheet.Post("/addinfo", wxSmall.InsertWXCSurveysheetResult)       //提交试卷
		wx_surveysheet.Post("/addinfocache", wxSmall.AddWxCacheSurveysheetResult) //提交试卷缓存
	}

}
